from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.db import transaction
from django.urls import path
from django.utils.translation import gettext_lazy as _

from . import views
from .urls import LEPRIKON_URL


@apphook_pool.register
class LeprikonApp(CMSApp):
    name = _("Leprikon")
    app_name = "leprikon"

    def get_urls(self, page=None, language=None, **kwargs):
        return ["leprikon.urls"]


@apphook_pool.register
class LeprikonActivityTypeApp(CMSApp):
    name = _("Leprikon activity type")
    app_name = "leprikon_activity_type"

    def get_urls(self, page=None, language=None, **kwargs):
        return [
            path("", views.activity_list, name="activity_list"),
            path("<int:pk>/", views.activity_detail, name="activity_detail"),
            path(
                "<int:pk>/{registration}/".format(**LEPRIKON_URL),
                transaction.atomic(views.registration_form),
                name="registration_form",
            ),
            path(
                "<int:pk>/<int:variant_pk>/{registration}/".format(**LEPRIKON_URL),
                transaction.atomic(views.registration_form),
                name="registration_form",
            ),
        ]


@apphook_pool.register
class LeprikonLeadersApp(CMSApp):
    name = _("Leprikon leaders")
    app_name = "leprikon_leaders"

    def get_urls(self, page=None, language=None, **kwargs):
        return [
            path("", views.leader_list, name="leader_list"),
            path("<slug>/", views.leader_detail, name="leader_detail"),
        ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-09 09:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0042_chat_group_types"),
    ]

    operations = [
        migrations.CreateModel(
            name="RegistrationLink",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("slug", models.SlugField(editable=False, max_length=64)),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                ("reg_from", models.DateTimeField(verbose_name="registration active from")),
                ("reg_to", models.DateTimeField(verbose_name="registration active to")),
                (
                    "school_year",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="registration_links",
                        to="leprikon.SchoolYear",
                        verbose_name="school year",
                    ),
                ),
                (
                    "subject_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="registration_links",
                        to="leprikon.SubjectType",
                        verbose_name="activity type",
                    ),
                ),
                (
                    "subjects",
                    models.ManyToManyField(
                        related_name="registration_links", to="leprikon.Subject", verbose_name="subjects"
                    ),
                ),
            ],
            options={
                "verbose_name": "registration link",
                "verbose_name_plural": "registration links",
                "ordering": ("reg_from", "reg_to"),
            },
        ),
        migrations.AddField(
            model_name="subjectregistration",
            name="registration_link",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="registrations",
                to="leprikon.RegistrationLink",
                verbose_name="registration link",
            ),
        ),
    ]



from django import forms
from django.contrib import admin
from django.http.response import HttpResponse, HttpResponseRedirect
from django.shortcuts import render
from django.urls import path, reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from ..forms.journals import JournalAdminForm, JournalEntryAdminForm, JournalLeaderEntryAdminForm
from ..models.activities import Activity
from ..models.journals import Journal, JournalEntry, JournalLeaderEntry, JournalTime
from ..utils import attributes
from .export import AdminExportMixin
from .filters import (
    ActivityListFilter,
    ActivityTypeListFilter,
    JournalListFilter,
    LeaderListFilter,
    SchoolYearListFilter,
)


class JournalTimeInlineAdmin(admin.TabularInline):
    model = JournalTime
    extra = 0


@admin.register(Journal)
class JournalAdmin(AdminExportMixin, admin.ModelAdmin):
    filter_horizontal = ("leaders", "participants")
    inlines = (JournalTimeInlineAdmin,)
    list_display = ("activity", "name", "get_times_list", "journal_links")
    list_filter = (
        ("activity__school_year", SchoolYearListFilter),
        ("activity__activity_type", ActivityTypeListFilter),
        ("leaders", LeaderListFilter),
        ("activity", ActivityListFilter),
    )
    raw_id_fields = ("activity",)

    # do not allow to add journals in admin
    def has_add_permission(self, request, obj=None):
        return False

    def changeform_view(self, request, object_id=None, form_url="", extra_context=None):
        if not object_id and request.method == "POST" and len(request.POST) == 3:
            return HttpResponseRedirect(f"{request.path}?activity=" + request.POST.get("activity", ""))
        else:
            return super().changeform_view(request, object_id, form_url, extra_context)

    def get_inline_instances(self, request, obj=None):
        return [] if hasattr(request, "hide_inlines") else super().get_inline_instances(request, obj)

    def get_form(self, request, obj, **kwargs):
        # get activity
        try:
            # first try request.POST (user may want to change activity)
            request.activity = Activity.objects.get(id=int(request.POST.get("activity")))
        except (Activity.DoesNotExist, TypeError, ValueError):
            if obj:
                # use activity from object
                request.activity = obj.activity
            else:
                # try to get activity from request.GET
                try:
                    request.activity = Activity.objects.get(
                        id=int(request.GET.get("activity")),
                    )
                except (Activity.DoesNotExist, TypeError, ValueError):
                    request.activity = None

        if request.activity:
            kwargs["form"] = type(
                "JournalAdminForm",
                (JournalAdminForm,),
                {"activity": request.activity},
            )
        else:
            kwargs["fields"] = ["activity"]
            request.hide_inlines = True

        return super().get_form(request, obj, **kwargs)

    def get_urls(self):
        return [
            path(
                "<int:journal_id>/journal/",
                self.admin_site.admin_view(self.journal),
                name="leprikon_journal_journal",
            ),
            path(
                "<int:journal_id>/journal-pdf/",
                self.admin_site.admin_view(self.journal_pdf),
                name="leprikon_journal_journal_pdf",
            ),
        ] + super().get_urls()

    def journal(self, request, journal_id):
        journal = self.get_object(request, journal_id)

        return render(
            request,
            "leprikon/journal_journal.html",
            {
                "journal": journal,
                "admin": True,
            },
        )

    def journal_pdf(self, request, journal_id):
        journal = self.get_object(request, journal_id)

        # create PDF response object
        response = HttpResponse(content_type="application/pdf")
        response["Content-Disposition"] = 'attachment; filename="{}"'.format(journal.get_pdf_filename("journal_pdf"))

        # write PDF to response
        return journal.write_pdf("journal_pdf", response)

    @attributes(short_description=_("journal"))
    def journal_links(self, obj):
        return mark_safe(
            "<br/>".join(
                (
                    format_html(
                        '<a href="{url}" title="{title}" target="_blank">{journal}</a>',
                        url=reverse("admin:leprikon_journal_journal", args=[obj.id]),
                        title=_("printable journal"),
                        journal=_("journal"),
                    ),
                    format_html(
                        '<a href="{url}" title="{title}" target="_blank">{participants}</a>',
                        url=reverse("admin:leprikon_journal_journal_pdf", args=[obj.id]),
                        title=_("printable list of participants"),
                        participants=_("participants"),
                    ),
                )
            )
        )


@admin.register(JournalLeaderEntry)
class JournalLeaderEntryAdmin(AdminExportMixin, admin.ModelAdmin):
    form = JournalLeaderEntryAdminForm
    list_display = ("timesheet", "date", "start", "end", "duration", "activity")
    list_filter = (
        ("timesheet__leader", LeaderListFilter),
        ("journal_entry__journal__activity", ActivityListFilter),
    )
    ordering = ("-journal_entry__date", "-start")

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj and obj.timesheet.submitted:
            return False
        return super().has_delete_permission(request, obj)

    def get_readonly_fields(self, request, obj=None):
        if obj and obj.timesheet.submitted:
            return ("start", "end")
        return self.readonly_fields


class JournalLeaderEntryInlineAdmin(admin.TabularInline):
    class form(forms.ModelForm):
        class Meta:
            model = JournalLeaderEntry
            fields = []

    model = JournalLeaderEntry
    ordering = ("journal_entry__date", "start")
    readonly_fields = ("date", "start", "end", "edit_link")

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj:
            # obj may be Timesheet or JournalEntry
            # this inline is used in both JournalEntryAdmin and TimesheetAdmin
            try:
                entries = obj.leader_entries
            except AttributeError:
                entries = obj.journal_entries
            if entries.filter(timesheet__submitted=True).exists():
                return False
        return super().has_delete_permission(request, obj)

    @attributes(short_description="")
    def edit_link(self, obj):
        return mark_safe(
            '<a href="{url}" title="{title}" target="_blank">{edit}</a>'.format(
                url=reverse("admin:leprikon_journalleaderentry_change", args=[obj.id]),
                title=_("update entry"),
                edit=_("edit"),
            )
        )


@admin.register(JournalEntry)
class JournalEntryAdmin(AdminExportMixin, admin.ModelAdmin):
    form = JournalEntryAdminForm
    date_hierarchy = "date"
    list_display = ("id", "journal_name", "date", "start", "end", "duration", "agenda_html")
    list_filter = (
        ("journal__activity__school_year", SchoolYearListFilter),
        ("journal__leaders", LeaderListFilter),
        ("journal", JournalListFilter),
    )
    filter_horizontal = ("participants", "participants_instructed")
    inlines = (JournalLeaderEntryInlineAdmin,)
    ordering = ("-date", "-start")
    readonly_fields = (
        "journal_name",
        "date",
    )

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj:
            if obj.affects_submitted_timesheets:
                return False
            else:
                return super().has_delete_permission(request, obj)
        return False

    def get_actions(self, request):
        actions = super().get_actions(request)
        if "delete_selected" in actions:

            def delete_selected(model_admin, request, queryset):
                objs = [
                    journal_entry for journal_entry in queryset.all() if not journal_entry.affects_submitted_timesheets
                ]
                return admin.actions.delete_selected(model_admin, request, objs)

            actions["delete_selected"] = (delete_selected, *actions["delete_selected"][1:])
        return actions

    @attributes(admin_order_field="journal", short_description=_("journal"))
    def journal_name(self, obj):
        return str(obj.journal)

    @attributes(admin_order_field="agenda", short_description=_("agenda"))
    def agenda_html(self, obj):
        return mark_safe(obj.agenda)



from collections import namedtuple

from django import forms
from django.forms.fields import MultiValueField
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

from ..conf import settings
from .widgets import RadioSelectBootstrap, RequiredOptionalWidget


class TextField(forms.CharField):
    widget = forms.Textarea


class AgreementBooleanField(forms.ChoiceField):
    def __init__(self, allow_disagree=False, **kwargs):
        empty_label = kwargs.pop("empty_label", "-")
        agree_label = kwargs.pop("agree_label", _("agree"))
        disagree_label = kwargs.pop("disagree_label", _("disagree"))
        kwargs["choices"] = (
            [
                ("", empty_label),
                (True, agree_label),
                (False, disagree_label),
            ]
            if allow_disagree
            else [
                ("", empty_label),
                (True, agree_label),
            ]
        )
        super().__init__(**kwargs)

    def to_python(self, value):
        if value == "True":
            value = True
        elif value == "False":
            value = False
        else:
            value = None
        return value


ReadonlyField = namedtuple("ReadonlyField", ("label", "value"))


class RequiredOptionalField(MultiValueField):
    def __init__(
        self,
        main_field,
        main_field_kwargs,
        choice_value_label=_("yes"),
        choice_empty_label=_("no"),
        choice_widget=None,
        widget=None,
        widget_attrs=None,
        **kwargs,
    ):
        self.empty_value = choice_empty_label

        # choice field
        choice_field = forms.ChoiceField(
            choices=(
                ("empty", choice_empty_label),
                ("value", choice_value_label),
            ),
            required=True,
            widget=choice_widget or RadioSelectBootstrap,
        )

        # main field
        main_field_kwargs["required"] = False
        main_field_class = import_string(settings.LEPRIKON_QUESTION_FIELDS[main_field]["class"])
        main_field = main_field_class(**main_field_kwargs)

        # widget
        widget = widget or RequiredOptionalWidget(
            (choice_field.widget, main_field.widget), widget_attrs, empty_value=choice_empty_label
        )

        super().__init__(
            (choice_field, main_field),
            require_all_fields=False,
            widget=widget,
            **kwargs,
        )

    def compress(self, data_list):
        choice, value = data_list
        if choice == "value":
            return value
        if choice == "empty":
            return str(self.empty_value)
        return None

    def clean(self, value):
        if value[0] == "value":
            self.fields[1].required = True
        return super().clean(value)



from django.urls import reverse_lazy as reverse
from django.utils.translation import gettext_lazy as _

from ..forms.parent import ParentForm
from ..models.roles import Parent
from .generic import CreateView, DeleteView, UpdateView


class ParentCreateView(CreateView):
    model = Parent
    form_class = ParentForm
    success_url = reverse("leprikon:summary")
    title = _("New parent")

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["user"] = self.request.user
        if self.request.user.leprikon_parents.count() == 0:
            kwargs["initial"] = dict(
                (attr, getattr(self.request.user, attr)) for attr in ["first_name", "last_name", "email"]
            )
        return kwargs

    def get_message(self):
        return _("New parent {} has been created.").format(self.object)


class ParentUpdateView(UpdateView):
    model = Parent
    form_class = ParentForm
    success_url = reverse("leprikon:summary")
    title = _("Change parent")

    def get_queryset(self):
        # only allow to edit user's own parents
        return super().get_queryset().filter(user=self.request.user)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["user"] = self.request.user
        return kwargs

    def get_message(self):
        return _("The parent {} has been updated.").format(self.object)


class ParentDeleteView(DeleteView):
    model = Parent
    title = _("Delete information about parent")
    message = _("Information about parent has been deleted.")

    def get_queryset(self):
        # only allow to delete user's own parents
        return super().get_queryset().filter(user=self.request.user)

    def get_question(self):
        return _("Do You really want to delete the information about parent: {}?").format(self.object)



from django.db import transaction
from django.urls import path
from django.views.generic.base import RedirectView

from . import views
from .conf import settings

# dictionary of all LEPRIKON_URL_* attributes of settings
LEPRIKON_URL = dict(
    (attr.lower()[len("LEPRIKON_URL_") :], getattr(settings, attr))
    for attr in dir(settings)
    if attr.startswith("LEPRIKON_URL_")
)


def d_path(pattern, name):
    return path(pattern.format(**LEPRIKON_URL), transaction.atomic(getattr(views, name)), name=name)


app_name = "leprikon"

urlpatterns = [
    path("", RedirectView.as_view(url="{summary}/".format(**LEPRIKON_URL), permanent=True), name="index"),
    d_path("{summary}/", "summary"),
    d_path("{create_account}/", "user_create"),
    d_path("{user}/{agreement}/", "user_agreement"),
    d_path("{user}/{email}/", "user_email"),
    d_path("{user}/", "user_update"),
    d_path("{user}/{password}/", "user_password"),
    d_path("{login}/", "user_login"),
    d_path("{logout}/", "user_logout"),
    d_path("{password_reset}/", "password_reset"),
    d_path("{password_reset}/done/", "password_reset_done"),
    d_path(
        "{password_reset}/<uidb64>/<token>/",
        "password_reset_confirm",
    ),
    d_path("{password_reset}/complete/", "password_reset_complete"),
    d_path("{leader}/", "leader_summary"),
    d_path("{leader}/{journal}/<int:pk>/", "journal"),
    d_path("{leader}/{journal}/{add}/<int:activity>/", "journal_create"),
    d_path("{leader}/{journal}/<int:pk>/{edit}/", "journal_update"),
    d_path("{leader}/{journal}/<int:pk>/{delete}/", "journal_delete"),
    d_path("{leader}/{journal}/{journalentry}/{add}/<int:journal>/", "journalentry_create"),
    d_path("{leader}/{journal}/{journalentry}/<int:pk>/", "journalentry_update"),
    d_path("{leader}/{journal}/{journalentry}/<int:pk>/{delete}/", "journalentry_delete"),
    d_path("{leader}/{journal}/{alternating}/", "alternating"),
    d_path("{leader}/{timesheets}/", "timesheet_list"),
    d_path("{leader}/{timesheets}/<int:pk>/", "timesheet_detail"),
    d_path("{leader}/{timesheets}/<int:pk>/{submit}/", "timesheet_submit"),
    d_path("{leader}/{timesheets}/{entry}/{add}/", "timesheetentry_create"),
    d_path("{leader}/{timesheets}/{entry}/<int:pk>/", "timesheetentry_update"),
    d_path("{leader}/{timesheets}/{entry}/<int:pk>/{delete}/", "timesheetentry_delete"),
    d_path("{leader}/{timesheets}/{journalentry}/<int:pk>/", "journalleaderentry_update"),
    d_path("{leader}/{timesheets}/{journalentry}/<int:pk>/{delete}/", "journalleaderentry_delete"),
    d_path("{leader}/<activity_type>/", "activity_list_mine"),
    d_path("{leader}/<activity_type>/<int:pk>/{edit}/", "activity_update"),
    d_path("{leader}/<activity_type>/<int:pk>/{journals}/", "activity_journals"),
    d_path("{leader}/<activity_type>/<int:pk>/{registrations}/", "activity_registrations"),
    d_path("{registrations}/", "registration_list"),
    d_path(
        "{registrations}/<int:pk>/{payment_request}-<variable_symbol>.pdf",
        "registration_payment_request",
    ),
    d_path("{registrations}/<int:pk>/<slug>.pdf", "registration_pdf"),
    d_path("{registrations}/<int:pk>/{cancel}/", "registration_cancel"),
    d_path("{registration_link}/<registration_link>/", "registration_link"),
    d_path("{registration_link}/<registration_link>/<int:pk>/", "registration_link_form"),
    d_path("{registration_link}/<registration_link>/<int:pk>/<int:variant_pk>/", "registration_link_form"),
    d_path("{payments}/", "payment_list"),
    d_path("{received_payments}/<int:pk>/<slug>.pdf", "received_payment_pdf"),
    d_path("{returned_payments}/<int:pk>/<slug>.pdf", "returned_payment_pdf"),
    d_path("{refund_requests}/<int:pk>/{add}/", "refund_request_create"),
    d_path("{refund_requests}/<int:pk>/", "refund_request_update"),
    d_path("{refund_requests}/<int:pk>/{delete}/", "refund_request_delete"),
    d_path("{payment_transfer}/<int:pk>/{add}/", "payment_transfer_create"),
    d_path("{donations}/<int:pk>/{add}/", "donation_create"),
    d_path("{donations}/", "donation_list"),
    d_path("{donations}/<pk>/<slug>.pdf", "donation_pdf"),
    d_path("{participants}/", "participant_list"),
    d_path("{participants}/{add}/", "participant_create"),
    d_path("{participants}/<int:pk>/", "participant_update"),
    d_path("{participants}/<int:pk>/{delete}/", "participant_delete"),
    d_path("{participants}/{parent}/{add}/", "parent_create"),
    d_path("{participants}/{parent}/<int:pk>/", "parent_update"),
    d_path("{participants}/{parent}/<int:pk>/{delete}/", "parent_delete"),
    d_path("{billing_info}/", "billing_info_list"),
    d_path("{billing_info}/{add}/", "billing_info_create"),
    d_path("{billing_info}/<int:pk>/", "billing_info_update"),
    d_path("{billing_info}/<int:pk>/{delete}/", "billing_info_delete"),
    d_path("{messages}/", "message_list"),
    d_path("{messages}/<slug>/", "message_detail"),
    d_path("{school_year}/", "school_year"),
    d_path("{reports}/", "report_list"),
    d_path("{reports}/{courses}/{payments}/", "report_course_payments"),
    d_path("{reports}/{courses}/{payments_status}/", "report_course_payments_status"),
    d_path("{reports}/{courses}/{stats}/", "report_course_stats"),
    d_path("{reports}/{events}/{payments}/", "report_event_payments"),
    d_path("{reports}/{events}/{payments_status}/", "report_event_payments_status"),
    d_path("{reports}/{events}/{stats}/", "report_event_stats"),
    d_path("{reports}/{orderables}/{payments}/", "report_orderable_payments"),
    d_path("{reports}/{orderables}/{payments_status}/", "report_orderable_payments_status"),
    d_path("{reports}/{orderables}/{stats}/", "report_orderable_stats"),
    d_path("{reports}/{debtors}/", "report_debtors"),
    d_path("{terms_conditions}/", "terms_conditions"),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2022-07-30 14:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields


def set_participant_counts(apps, schema_editor):
    Subject = apps.get_model("leprikon", "Subject")
    Subject.objects.filter(registration_type="G").update(
        min_participants_count=models.F("min_group_members_count"),
        max_participants_count=models.F("max_group_members_count"),
    )


def set_registration_schoolyear_division(apps, schema_editor):
    CourseRegistration = apps.get_model("leprikon", "CourseRegistration")
    for registration in CourseRegistration.objects.filter(school_year_division=None).annotate(
        variant_school_year_division_id=models.F("subject_variant__coursevariant__school_year_division_id"),
        subject_school_year_division_id=models.F("subject__course__school_year_division_id"),
    ):
        registration.school_year_division_id = (
            registration.variant_school_year_division_id or registration.subject_school_year_division_id
        )
        registration.save()


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0071_transaction_amount"),
    ]

    operations = [
        # SchoolYearDivision
        migrations.RenameField(
            model_name="schoolyeardivision",
            old_name="period_name",
            new_name="price_unit_name",
        ),
        migrations.AlterField(
            model_name="schoolyeardivision",
            name="price_unit_name",
            field=models.CharField(
                help_text="Unit name the price applies to. For example semester, month, or 5 lessons.",
                max_length=150,
                verbose_name="price unit name",
            ),
        ),
        # SchoolYearPeriod
        migrations.AddField(
            model_name="schoolyearperiod",
            name="price_units_count",
            field=models.DecimalField(decimal_places=2, default=1, max_digits=10, verbose_name="number of price units"),
        ),
        migrations.AlterField(
            model_name="schoolyearperiod",
            name="end",
            field=models.DateField(blank=True, null=True, verbose_name="end date"),
        ),
        migrations.AlterField(
            model_name="schoolyearperiod",
            name="start",
            field=models.DateField(blank=True, null=True, verbose_name="start date"),
        ),
        # Subject
        migrations.RunPython(set_participant_counts),
        migrations.RemoveField(
            model_name="subject",
            name="max_group_members_count",
        ),
        migrations.RemoveField(
            model_name="subject",
            name="min_group_members_count",
        ),
        migrations.AlterField(
            model_name="subject",
            name="max_participants_count",
            field=models.PositiveIntegerField(
                default=1, verbose_name="maximal participants / group members count per registration"
            ),
        ),
        migrations.AlterField(
            model_name="subject",
            name="min_participants_count",
            field=models.PositiveIntegerField(
                default=1, verbose_name="minimal participants / group members count per registration"
            ),
        ),
        migrations.RenameField(
            model_name="subject",
            old_name="price",
            new_name="registration_price",
        ),
        migrations.AlterField(
            model_name="subject",
            name="registration_price",
            field=leprikon.models.fields.PriceField(
                blank=True, decimal_places=0, max_digits=10, null=True, verbose_name="price per registration"
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="participant_price",
            field=leprikon.models.fields.PriceField(
                blank=True, decimal_places=0, max_digits=10, null=True, verbose_name="price per participant"
            ),
        ),
        # SubjectVariant
        migrations.AddField(
            model_name="subjectvariant",
            name="available",
            field=models.BooleanField(default=True, verbose_name="available for registration"),
        ),
        migrations.AddField(
            model_name="subjectvariant",
            name="age_groups",
            field=models.ManyToManyField(
                related_name="subject_variants", to="leprikon.AgeGroup", verbose_name="age groups"
            ),
        ),
        migrations.AddField(
            model_name="subjectvariant",
            name="target_groups",
            field=models.ManyToManyField(
                related_name="subject_variants", to="leprikon.TargetGroup", verbose_name="target groups"
            ),
        ),
        migrations.AddField(
            model_name="subjectvariant",
            name="min_participants_count",
            field=models.PositiveIntegerField(
                blank=True, null=True, verbose_name="minimal participants / group members count per registration"
            ),
        ),
        migrations.AddField(
            model_name="subjectvariant",
            name="max_participants_count",
            field=models.PositiveIntegerField(
                blank=True, null=True, verbose_name="maximal participants / group members count per registration"
            ),
        ),
        migrations.RenameField(
            model_name="subjectvariant",
            old_name="price",
            new_name="registration_price",
        ),
        migrations.AlterField(
            model_name="subjectvariant",
            name="registration_price",
            field=leprikon.models.fields.PriceField(
                blank=True, decimal_places=0, max_digits=10, null=True, verbose_name="price per registration"
            ),
        ),
        migrations.AddField(
            model_name="subjectvariant",
            name="participant_price",
            field=leprikon.models.fields.PriceField(
                blank=True, decimal_places=0, max_digits=10, null=True, verbose_name="price per participant"
            ),
        ),
        # CourseVariant
        migrations.CreateModel(
            name="CourseVariant",
            fields=[
                (
                    "subjectvariant_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="leprikon.SubjectVariant",
                    ),
                ),
                (
                    "allow_period_selection",
                    models.BooleanField(
                        default=False,
                        help_text="allow user to choose school year periods on registration form",
                        verbose_name="allow period selection",
                    ),
                ),
                (
                    "school_year_division",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="course_variants",
                        to="leprikon.SchoolYearDivision",
                        verbose_name="school year division",
                    ),
                ),
            ],
            bases=("leprikon.subjectvariant",),
        ),
        migrations.RunSQL(
            sql=(
                "INSERT INTO leprikon_coursevariant (subjectvariant_ptr_id, allow_period_selection, school_year_division_id) "
                "SELECT sv.id, c.allow_period_selection, c.school_year_division_id "
                "FROM leprikon_subjectvariant sv "
                "JOIN leprikon_course c ON c.subject_ptr_id = sv.subject_id;"
            ),
            reverse_sql="DELETE FROM leprikon_coursevariant;",
        ),
        # Course registration
        migrations.AddField(
            model_name="courseregistration",
            name="school_year_division",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="course_registrations",
                to="leprikon.SchoolYearDivision",
                verbose_name="school year division",
            ),
        ),
        migrations.RunPython(set_registration_schoolyear_division),
        migrations.AlterField(
            model_name="courseregistration",
            name="school_year_division",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="course_registrations",
                to="leprikon.SchoolYearDivision",
                verbose_name="school year division",
            ),
        ),
    ]



from django import forms
from django.utils.translation import gettext_lazy as _

from ...models.courses import Course
from ..form import FormMixin


class CoursePaymentsForm(FormMixin, forms.Form):
    date_start = forms.DateField(label=_("Start date"))
    date_end = forms.DateField(label=_("End date"))


class CoursePaymentsStatusForm(FormMixin, forms.Form):
    date = forms.DateField(label=_("Status for date"))


class CourseStatsForm(FormMixin, forms.Form):
    date = forms.DateField(label=_("Statistics for date"))
    courses = forms.ModelMultipleChoiceField(
        queryset=Course.objects.all(),
        label=_("Courses"),
    )
    paid_only = forms.BooleanField(label=_("Count only paid registrations"), required=False)
    paid_later = forms.BooleanField(label=_("Count registrations paid later"), required=False)
    approved_later = forms.BooleanField(label=_("Count registrations approved later"), required=False)
    unique_participants = forms.BooleanField(label=_("Count only unique participants"), required=False)
    max_weekly_hours = forms.FloatField(
        label=_("Maximal number of weekly hours"),
        help_text=_("Count only participants attending at most this number of hours weekly."),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        school_year = kwargs.pop("school_year")
        super().__init__(*args, **kwargs)
        self.fields["courses"].queryset = Course.objects.filter(school_year=school_year)



from django import forms
from django.core.exceptions import ValidationError
from django.utils import formats
from django.utils.translation import gettext_lazy as _

from ..models.timesheets import Timesheet, TimesheetEntry
from .fields import ReadonlyField
from .form import FormMixin


class TimesheetEntryAdminForm(forms.ModelForm):
    """
    Validation of ManyToManyField must be performed in form.clean()
    Always use TimesheetEntryAdminForm to change TimesheetEntry
    """

    class Meta:
        model = TimesheetEntry
        fields = ("date", "start", "end", "entry_type", "description")

    def __init__(self, **kwargs):
        self.leader = kwargs.pop("leader", None) or kwargs["instance"].timesheet.leader
        super().__init__(**kwargs)

        # make date, start and end read only, if timesheet is already submitted
        if self.instance.id and self.instance.timesheet.submitted:
            self.readonly = True
            self.readonly_fields = [
                ReadonlyField(label=_("Date"), value=self.instance.date),
                ReadonlyField(label=_("Start"), value=self.instance.start),
                ReadonlyField(label=_("End"), value=self.instance.end),
                ReadonlyField(label=_("Entry type"), value=self.instance.entry_type),
            ]
            self.fields = {"description": self.fields["description"]}
        else:
            self.readonly = False

    def clean_date(self):
        if self.instance.id:
            if self.cleaned_data["date"] < self.instance.timesheet.period.start:
                raise ValidationError(
                    _("The timesheet period {period} starts {start}").format(
                        period=self.instance.timesheet.period.name,
                        start=formats.date_format(self.instance.timesheet.period.start, "SHORT_DATE_FORMAT"),
                    )
                )
            if self.cleaned_data["date"] > self.instance.timesheet.period.end:
                raise ValidationError(
                    _("The timesheet period {period} ends {end}").format(
                        period=self.instance.timesheet.period.name,
                        end=formats.date_format(self.instance.timesheet.period.end, "SHORT_DATE_FORMAT"),
                    )
                )
        else:
            self.instance.timesheet = Timesheet.objects.for_leader_and_date(
                leader=self.leader, date=self.cleaned_data["date"]
            )
            if self.instance.timesheet.submitted:
                raise ValidationError(
                    _("Your timesheet for period {period} has already been submitted.").format(
                        period=self.instance.timesheet.period.name,
                    )
                )
        return self.cleaned_data["date"]

    def clean(self):
        # check overlaping entries
        if "date" in self.cleaned_data and "start" in self.cleaned_data and "end" in self.cleaned_data:
            qs = self.instance.timesheet.timesheet_entries.filter(
                date=self.cleaned_data["date"],
                start__lt=self.cleaned_data["end"],
                end__gt=self.cleaned_data["start"],
            )
            if self.instance.id:
                qs = qs.exclude(id=self.instance.id)
            if qs.exists():
                raise ValidationError(_("An overlaping entry has already been added in the timesheet."))
        return self.cleaned_data


class TimesheetEntryForm(FormMixin, TimesheetEntryAdminForm):
    pass



from django import forms
from django.contrib import admin
from django.db import transaction
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import path, reverse
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from ..forms.messages import MessageAdminForm
from ..models.messages import Message, MessageAttachment, MessageRecipient
from ..utils import attributes


def messagerecipient_send_mails(request, message, recipients, media):
    return render(
        request,
        "admin/leprikon/messagerecipient/send_mails.html",
        {
            "title": _("Sending emails"),
            "message": message,
            "recipients": recipients,
            "media": media,
            "message_opts": Message._meta,
            "opts": MessageRecipient._meta,
        },
    )


class SendMessageAdminMixin:
    actions = ("send_message", "add_to_message")

    def get_message_recipients(self, request, queryset):
        raise NotImplementedError("{} must implement method get_message_recipients".format(self.__class__.__name__))

    @attributes(short_description=_("Send message"))
    def send_message(self, request, queryset):
        request.method = "GET"
        request.leprikon_message_recipients = [r.id for r in self.get_message_recipients(request, queryset)]
        return admin.site._registry[Message].changeform_view(request, form_url=reverse("admin:leprikon_message_add"))

    @attributes(short_description=_("Add recipients to existing message"))
    def add_to_message(self, request, queryset):
        class MessageForm(forms.Form):
            message = forms.ModelChoiceField(
                label=_("Target message"),
                help_text=_("Recipients will be added to selected message."),
                queryset=Message.objects.all(),
            )

        if request.POST.get("post", "no") == "yes":
            form = MessageForm(request.POST)
            if form.is_valid():
                message = form.cleaned_data["message"]
                for recipient in self.get_message_recipients(request, queryset):
                    MessageRecipient.objects.get_or_create(message=message, recipient=recipient)
                return HttpResponseRedirect(
                    reverse("admin:leprikon_messagerecipient_changelist") + "?message={}".format(message.id)
                )
        else:
            form = MessageForm()
        return render(
            request,
            "admin/leprikon/message/add_recipients.html",
            {
                "title": _("Select target message"),
                "queryset": queryset,
                "opts": self.model._meta,
                "form": form,
                "action_checkbox_name": admin.helpers.ACTION_CHECKBOX_NAME,
            },
        )


class MessageAttachmentInlineAdmin(admin.TabularInline):
    model = MessageAttachment
    extra = 0


@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):
    form = MessageAdminForm
    inlines = (MessageAttachmentInlineAdmin,)
    list_display = ("subject", "created", "recipients", "action_links")
    search_fields = ("subject", "text")

    def get_changeform_initial_data(self, request):
        initial = super().get_changeform_initial_data(request)
        try:
            initial["recipients"] = request.leprikon_message_recipients
        except Exception:
            pass
        return initial

    @attributes(short_description=_("recipients"))
    def recipients(self, obj):
        return mark_safe(
            (
                '<a href="{recipients_url}">'
                '    <span title="{all_title}">{all_count}</span>'
                '  / <span title="{mails_title}">{mails_count}</span>'
                '  / <span title="{viewed_title}">{viewed_count}</span>'
                "</a> "
            ).format(
                recipients_url=reverse("admin:leprikon_messagerecipient_changelist") + "?message={}".format(obj.id),
                all_title=_("recipients count"),
                mails_title=_("sent mail"),
                viewed_title=_("viewed on site"),
                all_count=obj.recipients.count(),
                mails_count=obj.recipients.exclude(sent_mail=None).count(),
                viewed_count=obj.recipients.exclude(viewed=None).count(),
            )
        )

    @attributes(short_description=_("actions"))
    def action_links(self, obj):
        return mark_safe(
            (
                '<a href="{recipients_url}" class="button" title="{recipients_title}">{recipients}</a> '
                '<a href="{send_mails_all_url}" class="button" title="{send_mails_all_title}">{send_mails_all}</a> '
                '<a href="{send_mails_new_url}" class="button" title="{send_mails_new_title}">{send_mails_new}</a> '
            ).format(
                recipients=_("recipients"),
                recipients_title=_("show recipients details"),
                recipients_url=(reverse("admin:leprikon_messagerecipient_changelist") + "?message={}".format(obj.id)),
                send_mails_all=_("send mails"),
                send_mails_all_title=_("send mail to all recipients"),
                send_mails_all_url=reverse("admin:leprikon_message_send_mails") + "?message={}".format(obj.id),
                send_mails_new=_("send unsent"),
                send_mails_new_title=_("send mails to new recipients only"),
                send_mails_new_url=reverse("admin:leprikon_message_send_mails") + "?message={}&new=1".format(obj.id),
            )
        )

    def get_urls(self):
        return [
            path("send-mails/", self.admin_site.admin_view(self.send_mails), name="leprikon_message_send_mails"),
        ] + super().get_urls()

    def send_mails(self, request):
        try:
            message_id = int(request.GET["message"])
        except (KeyError, ValueError):
            return HttpResponseBadRequest()
        message = get_object_or_404(Message, id=message_id)
        recipients = message.recipients.all()
        if request.GET.get("new", False):
            recipients = recipients.filter(sent_mail=None)
        return messagerecipient_send_mails(request, message, recipients, self.media)


@admin.register(MessageRecipient)
class MessageRecipientAdmin(admin.ModelAdmin):
    list_display = ("recipient", "sent", "viewed", "sent_mail")
    actions = ("send_mails",)

    # hide the model from admin index
    def get_model_perms(self, request):
        return {}

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    @attributes(short_description=_("Send email to selected recipients"))
    def send_mails(self, request, queryset):
        return messagerecipient_send_mails(
            request,
            message=get_object_or_404(Message, id=request.GET.get("message", 0)),
            recipients=queryset,
            media=self.media,
        )

    def changelist_view(self, request, extra_context=None):
        message = get_object_or_404(Message, id=request.GET.get("message", 0))
        if extra_context is None:
            extra_context = {}
        extra_context.update(
            {
                "title": _("Recipients of message {}").format(message.subject),
                "message": message,
                "message_opts": Message._meta,
            }
        )
        return super().changelist_view(request, extra_context)

    def get_urls(self):
        return [
            path(
                "send-mail/",
                self.admin_site.admin_view(self.send_mail),
                name="leprikon_messagerecipient_send_mail",
            )
        ] + super().get_urls()

    @transaction.atomic
    def send_mail(self, request):
        try:
            recipient_id = int(request.GET["recipient_id"])
        except (KeyError, ValueError):
            return HttpResponseBadRequest()
        recipient = get_object_or_404(MessageRecipient, id=recipient_id)
        recipient.send_mail()
        return HttpResponse("0", content_type="text/json")



import calendar
from collections import namedtuple
from datetime import date, datetime, timedelta

from django.db import models
from django.urls import reverse_lazy as reverse
from django.utils.dateformat import DateFormat
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField

from ..utils import attributes
from .roles import Leader
from .startend import StartEndMixin


def start_end_by_date(d):
    # we work with monthly timesheets by default
    # TODO: allow weekly and quarterly timesheets by settings
    return {
        "start": date(d.year, d.month, 1),
        "end": date(d.year, d.month, calendar.monthrange(d.year, d.month)[1]),
    }


class TimesheetPeriodManager(models.Manager):
    def for_date(self, date):
        return self.get_or_create(**start_end_by_date(date))[0]


class TimesheetPeriod(StartEndMixin, models.Model):
    start = models.DateField(_("start date"), editable=False, unique=True)
    end = models.DateField(_("end date"), editable=False, unique=True)

    objects = TimesheetPeriodManager()

    class Meta:
        app_label = "leprikon"
        ordering = ("-start",)
        verbose_name = _("timesheet period")
        verbose_name_plural = _("timesheet periods")

    def __str__(self):
        return self.name

    @cached_property
    def name(self):
        # we work with monthly timesheets by default
        # TODO: allow weekly and quarterly timesheets by settings
        return DateFormat(self.start).format("F Y")

    @cached_property
    def all_timesheets(self):
        return list(self.timesheets.all())


class TimesheetManager(models.Manager):
    def for_leader_and_date(self, leader, date):
        return self.get_or_create(
            leader=leader,
            period=TimesheetPeriod.objects.for_date(date),
        )[0]


class Timesheet(models.Model):
    period = models.ForeignKey(
        TimesheetPeriod, verbose_name=_("period"), editable=False, related_name="timesheets", on_delete=models.PROTECT
    )
    leader = models.ForeignKey(
        Leader, verbose_name=_("leader"), editable=False, related_name="timesheets", on_delete=models.PROTECT
    )
    submitted = models.BooleanField(_("submitted"), default=False)
    paid = models.BooleanField(_("paid"), default=False)

    objects = TimesheetManager()

    class Meta:
        app_label = "leprikon"
        ordering = ("-period__start", "leader")
        unique_together = (("period", "leader"),)
        verbose_name = _("timesheet")
        verbose_name_plural = _("timesheets")

    def __str__(self):
        return "{leader}, {period}".format(
            leader=self.leader,
            period=self.period.name,
        )

    @cached_property
    def all_journal_entries(self):
        return list(self.journal_entries.all())

    @cached_property
    def all_timesheet_entries(self):
        return list(self.timesheet_entries.all())

    @cached_property
    def all_entries(self):
        return sorted(
            self.all_timesheet_entries + self.all_journal_entries,
            key=lambda e: e.datetime_start,
        )

    class EntryGroup(namedtuple("_EntryGroup", ("name", "entries"))):
        @property
        def duration(self):
            return sum((e.duration for e in self.entries), timedelta())

    @cached_property
    def groups(self):
        gs = {}
        for entry in self.all_entries:
            if entry.group not in gs:
                gs[entry.group] = self.EntryGroup(
                    name=entry.group.name,
                    entries=[],
                )
            gs[entry.group].entries.append(entry)
        return list(gs.values())


class TimesheetEntryType(models.Model):
    name = models.CharField(_("name"), max_length=150)
    order = models.IntegerField(_("order"), blank=True, default=0)

    class Meta:
        app_label = "leprikon"
        ordering = ("order",)
        verbose_name = _("timesheet entry type")
        verbose_name_plural = _("timesheet entry types")

    def __str__(self):
        return self.name


class TimesheetEntry(StartEndMixin, models.Model):
    timesheet = models.ForeignKey(
        Timesheet,
        verbose_name=_("timesheet"),
        editable=False,
        related_name="timesheet_entries",
        on_delete=models.PROTECT,
    )
    entry_type = models.ForeignKey(
        TimesheetEntryType, verbose_name=_("entry type"), null=True, related_name="entries", on_delete=models.PROTECT
    )
    date = models.DateField(_("date"))
    start = models.TimeField(_("start time"))
    end = models.TimeField(_("end time"))
    description = HTMLField(_("work description"))

    class Meta:
        app_label = "leprikon"
        ordering = ("start",)
        verbose_name = _("timesheet entry")
        verbose_name_plural = _("timesheet entries")

    def __str__(self):
        return "{}".format(self.duration)

    @cached_property
    def datetime_start(self):
        return datetime.combine(self.date, self.start)

    @cached_property
    def datetime_end(self):
        return datetime.combine(self.date, self.end)

    @attributes(short_description=_("duration"))
    @cached_property
    def duration(self):
        return self.datetime_end - self.datetime_start

    @property
    def group(self):
        return self.entry_type

    def get_edit_url(self):
        return reverse("leprikon:timesheetentry_update", args=(self.id,))

    def get_delete_url(self):
        return reverse("leprikon:timesheetentry_delete", args=(self.id,))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-08-15 15:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0063_journals"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="journalentry",
            name="subject",
        ),
        migrations.RemoveField(
            model_name="subject",
            name="evaluation",
        ),
        migrations.RemoveField(
            model_name="subject",
            name="plan",
        ),
        migrations.RemoveField(
            model_name="subject",
            name="risks",
        ),
        migrations.AlterField(
            model_name="journalentry",
            name="journal",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="journal_entries",
                to="leprikon.Journal",
                verbose_name="journal",
            ),
        ),
    ]



from functools import cached_property
from typing import List

from django.contrib import admin
from django.db.models import Model
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _


class ParentAdminMixin:
    child_models: List[Model] = []

    @cached_property
    def child_admins(self) -> List[admin.ModelAdmin]:
        return [admin.site._registry.get(child_model) for child_model in self.child_models]

    def get_list_display(self: admin.ModelAdmin, request):
        list_display = list(super().get_list_display(request))
        permissions = [admin_instance.has_change_permission(request) for admin_instance in self.child_admins]

        if any(permissions):

            preserved_filters = self.get_preserved_filters(request)

            @admin.display(description=_("settings"))
            def child_links(obj):
                return mark_safe(
                    " | ".join(
                        format_html(
                            '<a href="{}">{}</a>',
                            "%s?%s"
                            % (
                                reverse(
                                    f"admin:{child_model._meta.app_label}_{child_model._meta.model_name}_change",
                                    args=(obj.id,),
                                ),
                                preserved_filters,
                            ),
                            child_model._meta.verbose_name_plural,
                        )
                        for child_model, permission in zip(self.child_models, permissions)
                        if permission
                    )
                )

            list_display.append(child_links)

        return list_display


class ChildModelAdmin(admin.ModelAdmin):
    parent_model: Model

    @cached_property
    def parent_admin(self) -> admin.ModelAdmin:
        return admin.site._registry.get(self.parent_model)

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def has_view_permission(self, request, obj=None):
        return False

    def has_module_permission(self, request):
        return False

    # redirect to parent admin after save
    def response_post_save_change(self, request, obj):
        return self.parent_admin.response_post_save_change(request, obj)



# Generated by Django 1.9.9 on 2017-02-06 02:43

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion
import django_countries.fields
import djangocms_text_ckeditor.fields
import filer.fields.file
import filer.fields.image
from leprikon.conf import settings
import leprikon.models.fields
import leprikon.models.startend
import localflavor.generic.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("cms", "0001_initial"),
        ("filer", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="AgeGroup",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "age group",
                "verbose_name_plural": "age groups",
            },
        ),
        migrations.CreateModel(
            name="Contact",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "contact_type",
                    models.CharField(
                        choices=settings.LEPRIKON_CONTACT_TYPES, max_length=30, verbose_name="contact type"
                    ),
                ),
                ("contact", models.CharField(max_length=250, verbose_name="contact")),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
                ("public", models.BooleanField(default=False, verbose_name="public")),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "contact",
                "verbose_name_plural": "contacts",
            },
        ),
        migrations.CreateModel(
            name="CourseDiscount",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="time of discount")),
                (
                    "amount",
                    leprikon.models.fields.PriceField(
                        decimal_places=0, default=0, max_digits=10, verbose_name="discount"
                    ),
                ),
                (
                    "explanation",
                    models.CharField(blank=True, default="", max_length=250, verbose_name="discount explanation"),
                ),
            ],
            options={
                "ordering": ("created",),
                "abstract": False,
                "verbose_name": "discount",
                "verbose_name_plural": "discounts",
            },
        ),
        migrations.CreateModel(
            name="CourseJournalEntry",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("date", models.DateField(verbose_name="date")),
                ("start", models.TimeField(blank=True, null=True, verbose_name="start time")),
                ("end", models.TimeField(blank=True, null=True, verbose_name="end time")),
                ("agenda", djangocms_text_ckeditor.fields.HTMLField(default="", verbose_name="session agenda")),
            ],
            options={
                "ordering": ("date", "start", "end"),
                "verbose_name": "journal entry",
                "verbose_name_plural": "journal entries",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CourseJournalLeaderEntry",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("start", models.TimeField(verbose_name="start time")),
                ("end", models.TimeField(verbose_name="end time")),
                (
                    "course_entry",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="leader_entries",
                        to="leprikon.CourseJournalEntry",
                        verbose_name="course journal entry",
                    ),
                ),
            ],
            options={
                "verbose_name": "course journal leader entry",
                "verbose_name_plural": "course journal leader entries",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CourseListPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_courselistplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                (
                    "template",
                    models.CharField(
                        choices=[("default", "Default"), ("grouped", "Grouped by course groups")],
                        default="default",
                        help_text="The template used to render plugin.",
                        max_length=100,
                        verbose_name="template",
                    ),
                ),
                (
                    "age_groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Keep empty to skip searching by age groups.",
                        related_name="+",
                        to="leprikon.AgeGroup",
                        verbose_name="age groups",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="CoursePlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_courseplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                (
                    "template",
                    models.CharField(
                        choices=[("default", "Default")],
                        default="default",
                        help_text="The template used to render plugin.",
                        max_length=100,
                        verbose_name="template",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="CourseRegistrationHistory",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("start", models.DateField()),
                ("end", models.DateField(blank=True, null=True)),
            ],
            options={
                "ordering": ("start",),
                "verbose_name": "course registration history",
                "verbose_name_plural": "course registration history",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CourseTime",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "day_of_week",
                    models.IntegerField(
                        choices=[
                            (1, "Monday"),
                            (2, "Tuesday"),
                            (3, "Wednesday"),
                            (4, "Thursday"),
                            (5, "Friday"),
                            (6, "Saturday"),
                            (7, "Sunday"),
                        ],
                        verbose_name="day of week",
                    ),
                ),
                ("start", models.TimeField(blank=True, null=True, verbose_name="start time")),
                ("end", models.TimeField(blank=True, null=True, verbose_name="end time")),
            ],
            options={
                "ordering": ("day_of_week", "start"),
                "verbose_name": "time",
                "verbose_name_plural": "times",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="EventDiscount",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="time of discount")),
                (
                    "amount",
                    leprikon.models.fields.PriceField(
                        decimal_places=0, default=0, max_digits=10, verbose_name="discount"
                    ),
                ),
                (
                    "explanation",
                    models.CharField(blank=True, default="", max_length=250, verbose_name="discount explanation"),
                ),
            ],
            options={
                "ordering": ("created",),
                "abstract": False,
                "verbose_name": "discount",
                "verbose_name_plural": "discounts",
            },
        ),
        migrations.CreateModel(
            name="EventListPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_eventlistplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                (
                    "template",
                    models.CharField(
                        choices=[("default", "Default"), ("grouped", "Grouped by event groups")],
                        default="default",
                        help_text="The template used to render plugin.",
                        max_length=100,
                        verbose_name="template",
                    ),
                ),
                (
                    "age_groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Keep empty to skip searching by age groups.",
                        related_name="+",
                        to="leprikon.AgeGroup",
                        verbose_name="age groups",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="EventPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_eventplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                (
                    "template",
                    models.CharField(
                        choices=[("default", "Default")],
                        default="default",
                        help_text="The template used to render plugin.",
                        max_length=100,
                        verbose_name="template",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="FilteredCourseListPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_filteredcourselistplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="FilteredEventListPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_filteredeventlistplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="FilteredLeaderListPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_filteredleaderlistplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Insurance",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("code", models.CharField(max_length=10, unique=True, verbose_name="code")),
                ("name", models.CharField(max_length=150, verbose_name="name")),
            ],
            options={
                "ordering": ("code",),
                "verbose_name": "insurance company",
                "verbose_name_plural": "insurance companies",
            },
        ),
        migrations.CreateModel(
            name="Leader",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "description",
                    djangocms_text_ckeditor.fields.HTMLField(blank=True, default="", verbose_name="description"),
                ),
                (
                    "page",
                    cms.models.fields.PageField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="cms.Page",
                        verbose_name="page",
                    ),
                ),
                (
                    "photo",
                    filer.fields.image.FilerImageField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="filer.Image",
                        verbose_name="photo",
                    ),
                ),
            ],
            options={
                "ordering": ("user__last_name", "user__first_name"),
                "verbose_name": "leader",
                "verbose_name_plural": "leaders",
            },
        ),
        migrations.CreateModel(
            name="LeaderListPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_leaderlistplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                (
                    "template",
                    models.CharField(
                        choices=[("default", "Default")],
                        default="default",
                        help_text="The template used to render plugin.",
                        max_length=100,
                        verbose_name="template",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="LeaderPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="leprikon_leaderplugin",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                (
                    "template",
                    models.CharField(
                        choices=[("default", "Default")],
                        default="default",
                        help_text="The template used to render plugin.",
                        max_length=100,
                        verbose_name="template",
                    ),
                ),
                (
                    "leader",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="leprikon.Leader",
                        verbose_name="leader",
                    ),
                ),
            ],
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="LeprikonSite",
            fields=[
                (
                    "site_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="sites.Site",
                    ),
                ),
                ("city", models.CharField(blank=True, max_length=150, null=True, verbose_name="city")),
                (
                    "postal_code",
                    leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name="postal code"),
                ),
                (
                    "email",
                    leprikon.models.fields.EmailField(
                        blank=True, max_length=254, null=True, verbose_name="email address"
                    ),
                ),
                ("phone", models.CharField(blank=True, max_length=30, null=True, verbose_name="phone")),
                ("street", models.CharField(blank=True, max_length=150, null=True, verbose_name="street")),
                ("company_num", models.CharField(blank=True, max_length=8, null=True, verbose_name="company number")),
                ("vat_number", models.CharField(blank=True, max_length=10, null=True, verbose_name="VAT number")),
                ("iban", localflavor.generic.models.IBANField("IBAN", None, blank=True, null=True)),
                ("bic", localflavor.generic.models.BICField(blank=True, null=True, verbose_name="BIC (SWIFT)")),
                ("agreement", models.TextField(blank=True, null=True, verbose_name="registration agreement")),
            ],
            options={
                "verbose_name": "leprikon site",
                "verbose_name_plural": "leprikon sites",
            },
            bases=("sites.site",),
        ),
        migrations.CreateModel(
            name="Message",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="created")),
                ("subject", models.CharField(max_length=150, verbose_name="subject")),
                ("text", djangocms_text_ckeditor.fields.HTMLField(default="", verbose_name="text")),
            ],
            options={
                "ordering": ("-created",),
                "verbose_name": "message",
                "verbose_name_plural": "messages",
            },
        ),
        migrations.CreateModel(
            name="MessageAttachment",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "file",
                    filer.fields.file.FilerFileField(
                        on_delete=django.db.models.deletion.CASCADE, related_name="+", to="filer.File"
                    ),
                ),
                (
                    "message",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attachments",
                        to="leprikon.Message",
                        verbose_name="message",
                    ),
                ),
            ],
            options={
                "verbose_name": "attachment",
                "verbose_name_plural": "attachments",
            },
        ),
        migrations.CreateModel(
            name="MessageRecipient",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("slug", models.SlugField(editable=False)),
                ("sent", models.DateTimeField(auto_now_add=True, verbose_name="sent")),
                (
                    "viewed",
                    models.DateTimeField(default=None, editable=False, null=True, verbose_name="viewed on site"),
                ),
                (
                    "sent_mail",
                    models.DateTimeField(default=None, editable=False, null=True, verbose_name="sent by email"),
                ),
                (
                    "message",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="recipients",
                        to="leprikon.Message",
                        verbose_name="message",
                    ),
                ),
                (
                    "recipient",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="leprikon_messages",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="recipient",
                    ),
                ),
            ],
            options={
                "ordering": ("-sent",),
                "verbose_name": "recipient",
                "verbose_name_plural": "recipients",
            },
        ),
        migrations.CreateModel(
            name="Parent",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("first_name", models.CharField(max_length=30, verbose_name="first name")),
                ("last_name", models.CharField(max_length=30, verbose_name="last name")),
                ("street", models.CharField(max_length=150, verbose_name="street")),
                ("city", models.CharField(max_length=150, verbose_name="city")),
                ("postal_code", leprikon.models.fields.PostalCodeField(verbose_name="postal code")),
                (
                    "email",
                    leprikon.models.fields.EmailField(
                        blank=True, default="", max_length=254, verbose_name="email address"
                    ),
                ),
                ("phone", models.CharField(max_length=30, verbose_name="phone")),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="leprikon_parents",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "parent",
                "verbose_name_plural": "parents",
            },
        ),
        migrations.CreateModel(
            name="Participant",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("first_name", models.CharField(max_length=30, verbose_name="first name")),
                ("last_name", models.CharField(max_length=30, verbose_name="last name")),
                ("birth_num", leprikon.models.fields.BirthNumberField(verbose_name="birth number")),
                ("street", models.CharField(max_length=150, verbose_name="street")),
                ("city", models.CharField(max_length=150, verbose_name="city")),
                ("postal_code", leprikon.models.fields.PostalCodeField(verbose_name="postal code")),
                (
                    "email",
                    leprikon.models.fields.EmailField(
                        blank=True, default="", max_length=254, verbose_name="email address"
                    ),
                ),
                ("phone", models.CharField(blank=True, default="", max_length=30, verbose_name="phone")),
                ("citizenship", django_countries.fields.CountryField(max_length=2, verbose_name="citizenship")),
                ("school_other", models.CharField(blank=True, default="", max_length=150, verbose_name="other school")),
                ("school_class", models.CharField(blank=True, default="", max_length=30, verbose_name="class")),
                ("health", models.TextField(blank=True, default="", verbose_name="health")),
                (
                    "gender",
                    models.CharField(
                        choices=[("m", "male / boy"), ("f", "female / girl")],
                        editable=False,
                        max_length=1,
                        verbose_name="gender",
                    ),
                ),
                (
                    "age_group",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="leprikon.AgeGroup",
                        verbose_name="age group",
                    ),
                ),
                (
                    "insurance",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="leprikon.Insurance",
                        verbose_name="insurance",
                    ),
                ),
            ],
            options={
                "verbose_name": "participant",
                "verbose_name_plural": "participants",
            },
        ),
        migrations.CreateModel(
            name="Place",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                ("place", models.CharField(blank=True, default="", max_length=50, verbose_name="place")),
            ],
            options={
                "ordering": ("place",),
                "verbose_name": "place",
                "verbose_name_plural": "places",
            },
        ),
        migrations.CreateModel(
            name="PrintSetup",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=150, verbose_name="name")),
                (
                    "top",
                    models.IntegerField(
                        blank=True, default=20, help_text="distance in milimetres", verbose_name="margin top"
                    ),
                ),
                (
                    "left",
                    models.IntegerField(
                        blank=True, default=20, help_text="distance in milimetres", verbose_name="margin left"
                    ),
                ),
                (
                    "right",
                    models.IntegerField(
                        blank=True, default=20, help_text="distance in milimetres", verbose_name="margin right"
                    ),
                ),
                (
                    "bottom",
                    models.IntegerField(
                        blank=True, default=20, help_text="distance in milimetres", verbose_name="margin bottom"
                    ),
                ),
                (
                    "background",
                    filer.fields.file.FilerFileField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="filer.File",
                        verbose_name="pdf background",
                    ),
                ),
            ],
            options={
                "ordering": ("name",),
                "verbose_name": "print setup",
                "verbose_name_plural": "print setups",
            },
        ),
        migrations.CreateModel(
            name="Question",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=50, unique=True, verbose_name="name")),
                ("question", models.CharField(max_length=50, verbose_name="question")),
                (
                    "help_text",
                    models.TextField(
                        blank=True,
                        help_text="This is help text. The help text is shown next to the form field.",
                        null=True,
                        verbose_name="help text",
                    ),
                ),
                (
                    "field",
                    models.CharField(
                        choices=((key, val["name"]) for key, val in settings.LEPRIKON_QUESTION_FIELDS.items()),
                        max_length=150,
                        verbose_name="field",
                    ),
                ),
                (
                    "field_args",
                    models.TextField(
                        blank=True,
                        default="{}",
                        help_text="Enter valid JSON structure representing field configuration.",
                        verbose_name="field_args",
                    ),
                ),
            ],
            options={
                "verbose_name": "additional question",
                "verbose_name_plural": "additional questions",
            },
        ),
        migrations.CreateModel(
            name="School",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                ("street", models.CharField(blank=True, default="", max_length=50, verbose_name="street")),
                ("city", models.CharField(blank=True, default="", max_length=50, verbose_name="city")),
            ],
            options={
                "ordering": ("city", "name"),
                "verbose_name": "school",
                "verbose_name_plural": "schools",
            },
        ),
        migrations.CreateModel(
            name="SchoolYear",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "year",
                    models.IntegerField(help_text="Only the first of the two years.", unique=True, verbose_name="year"),
                ),
                ("active", models.BooleanField(default=False, verbose_name="active")),
            ],
            options={
                "ordering": ("-year",),
                "verbose_name": "school year",
                "verbose_name_plural": "school years",
            },
        ),
        migrations.CreateModel(
            name="SchoolYearPeriod",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=150, verbose_name="name")),
                ("start", models.DateField(verbose_name="start date")),
                ("end", models.DateField(verbose_name="end date")),
                (
                    "school_year",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="periods",
                        to="leprikon.SchoolYear",
                        verbose_name="school year",
                    ),
                ),
            ],
            options={
                "ordering": ("start",),
                "verbose_name": "school year period",
                "verbose_name_plural": "school year periods",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="Subject",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=150, verbose_name="name")),
                (
                    "description",
                    djangocms_text_ckeditor.fields.HTMLField(blank=True, default="", verbose_name="description"),
                ),
                (
                    "price",
                    leprikon.models.fields.PriceField(
                        blank=True, decimal_places=0, max_digits=10, null=True, verbose_name="price"
                    ),
                ),
                ("public", models.BooleanField(default=False, verbose_name="public")),
                ("reg_from", models.DateTimeField(blank=True, null=True, verbose_name="registration active from")),
                ("reg_to", models.DateTimeField(blank=True, null=True, verbose_name="registration active to")),
                ("min_count", models.IntegerField(blank=True, null=True, verbose_name="minimal count")),
                ("max_count", models.IntegerField(blank=True, null=True, verbose_name="maximal count")),
                ("risks", djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name="risks")),
                ("plan", djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name="plan")),
                ("evaluation", djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name="evaluation")),
                ("note", models.CharField(blank=True, default="", max_length=300, verbose_name="note")),
                ("agreement", models.TextField(blank=True, null=True, verbose_name="registration agreement")),
            ],
            options={
                "ordering": ("name",),
                "verbose_name": "subject",
                "verbose_name_plural": "subjects",
            },
        ),
        migrations.CreateModel(
            name="SubjectAttachment",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
                (
                    "file",
                    filer.fields.file.FilerFileField(
                        on_delete=django.db.models.deletion.CASCADE, related_name="+", to="filer.File"
                    ),
                ),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "attachment",
                "verbose_name_plural": "attachments",
            },
        ),
        migrations.CreateModel(
            name="SubjectGroup",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=150, verbose_name="name")),
                ("plural", models.CharField(max_length=150, verbose_name="plural")),
                ("color", leprikon.models.fields.ColorField(max_length=10, verbose_name="color")),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "subject group",
                "verbose_name_plural": "subject groups",
            },
        ),
        migrations.CreateModel(
            name="SubjectPayment",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="payment time")),
                (
                    "payment_type",
                    models.CharField(
                        choices=[
                            ("PAYMENT_CASH", "payment - cash"),
                            ("PAYMENT_BANK", "payment - bank"),
                            ("PAYMENT_ONLINE", "payment - online"),
                            ("PAYMENT_TRANSFER", "payment - transfer from return"),
                            ("RETURN_CASH", "return - cash"),
                            ("RETURN_BANK", "return - bank"),
                            ("RETURN_TRANSFER", "return - transfer to payment"),
                        ],
                        max_length=30,
                        verbose_name="payment type",
                    ),
                ),
                (
                    "amount",
                    leprikon.models.fields.PriceField(
                        decimal_places=0,
                        help_text="positive value for payment, negative value for return",
                        max_digits=10,
                        verbose_name="amount",
                    ),
                ),
                ("note", models.CharField(blank=True, default="", max_length=300, verbose_name="note")),
            ],
            options={
                "ordering": ("created",),
                "verbose_name": "payment",
                "verbose_name_plural": "payments",
            },
        ),
        migrations.CreateModel(
            name="SubjectRegistration",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("slug", models.SlugField(editable=False)),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="time of registration")),
                (
                    "price",
                    leprikon.models.fields.PriceField(
                        decimal_places=0, editable=False, max_digits=10, verbose_name="price"
                    ),
                ),
                (
                    "answers",
                    models.TextField(blank=True, default="{}", editable=False, verbose_name="additional answers"),
                ),
                ("approved", models.DateTimeField(editable=False, null=True, verbose_name="time of approval")),
                ("canceled", models.DateTimeField(editable=False, null=True, verbose_name="time of cancellation")),
                ("cancel_request", models.BooleanField(default=False, verbose_name="cancel request")),
                (
                    "participant_gender",
                    models.CharField(
                        choices=[("m", "male"), ("f", "female")], editable=False, max_length=1, verbose_name="gender"
                    ),
                ),
                ("participant_first_name", models.CharField(max_length=30, verbose_name="first name")),
                ("participant_last_name", models.CharField(max_length=30, verbose_name="last name")),
                ("participant_birth_num", leprikon.models.fields.BirthNumberField(verbose_name="birth number")),
                ("participant_street", models.CharField(max_length=150, verbose_name="street")),
                ("participant_city", models.CharField(max_length=150, verbose_name="city")),
                ("participant_postal_code", leprikon.models.fields.PostalCodeField(verbose_name="postal code")),
                (
                    "participant_citizenship",
                    django_countries.fields.CountryField(max_length=2, verbose_name="citizenship"),
                ),
                ("participant_phone", models.CharField(blank=True, default="", max_length=30, verbose_name="phone")),
                (
                    "participant_email",
                    leprikon.models.fields.EmailField(
                        blank=True, default="", max_length=254, verbose_name="email address"
                    ),
                ),
                (
                    "participant_school_other",
                    models.CharField(blank=True, default="", max_length=150, verbose_name="other school"),
                ),
                (
                    "participant_school_class",
                    models.CharField(blank=True, default="", max_length=30, verbose_name="class"),
                ),
                ("participant_health", models.TextField(blank=True, default="", verbose_name="health")),
                ("has_parent1", models.BooleanField(default=False, verbose_name="first parent")),
                (
                    "parent1_first_name",
                    models.CharField(blank=True, max_length=30, null=True, verbose_name="first name"),
                ),
                ("parent1_last_name", models.CharField(blank=True, max_length=30, null=True, verbose_name="last name")),
                ("parent1_street", models.CharField(blank=True, max_length=150, null=True, verbose_name="street")),
                ("parent1_city", models.CharField(blank=True, max_length=150, null=True, verbose_name="city")),
                (
                    "parent1_postal_code",
                    leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name="postal code"),
                ),
                ("parent1_phone", models.CharField(blank=True, max_length=30, null=True, verbose_name="phone")),
                (
                    "parent1_email",
                    leprikon.models.fields.EmailField(
                        blank=True, max_length=254, null=True, verbose_name="email address"
                    ),
                ),
                ("has_parent2", models.BooleanField(default=False, verbose_name="second parent")),
                (
                    "parent2_first_name",
                    models.CharField(blank=True, max_length=30, null=True, verbose_name="first name"),
                ),
                ("parent2_last_name", models.CharField(blank=True, max_length=30, null=True, verbose_name="last name")),
                ("parent2_street", models.CharField(blank=True, max_length=150, null=True, verbose_name="street")),
                ("parent2_city", models.CharField(blank=True, max_length=150, null=True, verbose_name="city")),
                (
                    "parent2_postal_code",
                    leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name="postal code"),
                ),
                ("parent2_phone", models.CharField(blank=True, max_length=30, null=True, verbose_name="phone")),
                (
                    "parent2_email",
                    leprikon.models.fields.EmailField(
                        blank=True, max_length=254, null=True, verbose_name="email address"
                    ),
                ),
            ],
            options={
                "verbose_name": "registration",
                "verbose_name_plural": "registrations",
            },
        ),
        migrations.CreateModel(
            name="SubjectType",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "subject_type",
                    models.CharField(
                        choices=[("course", "course"), ("event", "event")], max_length=10, verbose_name="subjects"
                    ),
                ),
                ("name", models.CharField(max_length=150, verbose_name="name (singular)")),
                ("name_genitiv", models.CharField(blank=True, max_length=150, verbose_name="name (genitiv)")),
                ("name_akuzativ", models.CharField(blank=True, max_length=150, verbose_name="name (akuzativ)")),
                ("plural", models.CharField(max_length=150, verbose_name="name (plural)")),
                ("slug", models.SlugField()),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
                (
                    "questions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Add additional questions to be asked in the registration form.",
                        related_name="+",
                        to="leprikon.Question",
                        verbose_name="additional questions",
                    ),
                ),
                ("agreement", models.TextField(blank=True, null=True, verbose_name="registration agreement")),
                (
                    "reg_print_setup",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="leprikon.PrintSetup",
                        verbose_name="registration print setup",
                    ),
                ),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "subject type",
                "verbose_name_plural": "subject types",
            },
        ),
        migrations.CreateModel(
            name="SubjectTypeAttachment",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
                (
                    "file",
                    filer.fields.file.FilerFileField(
                        on_delete=django.db.models.deletion.CASCADE, related_name="+", to="filer.File"
                    ),
                ),
                (
                    "subject_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attachments",
                        to="leprikon.SubjectType",
                        verbose_name="subject type",
                    ),
                ),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "attachment",
                "verbose_name_plural": "attachments",
            },
        ),
        migrations.CreateModel(
            name="Timesheet",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("submitted", models.BooleanField(default=False, verbose_name="submitted")),
                ("paid", models.BooleanField(default=False, verbose_name="paid")),
                (
                    "leader",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="timesheets",
                        to="leprikon.Leader",
                        verbose_name="leader",
                    ),
                ),
            ],
            options={
                "ordering": ("-period__start", "leader"),
                "verbose_name": "timesheet",
                "verbose_name_plural": "timesheets",
            },
        ),
        migrations.CreateModel(
            name="TimesheetEntry",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("date", models.DateField(verbose_name="date")),
                ("start", models.TimeField(verbose_name="start time")),
                ("end", models.TimeField(verbose_name="end time")),
                ("description", djangocms_text_ckeditor.fields.HTMLField(verbose_name="work description")),
            ],
            options={
                "ordering": ("start",),
                "verbose_name": "timesheet entry",
                "verbose_name_plural": "timesheet entries",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="TimesheetEntryType",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=150, verbose_name="name")),
                ("order", models.IntegerField(blank=True, default=0, verbose_name="order")),
            ],
            options={
                "ordering": ("order",),
                "verbose_name": "timesheet entry type",
                "verbose_name_plural": "timesheet entry types",
            },
        ),
        migrations.CreateModel(
            name="TimesheetPeriod",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("start", models.DateField(editable=False, unique=True, verbose_name="start date")),
                ("end", models.DateField(editable=False, unique=True, verbose_name="end date")),
            ],
            options={
                "ordering": ("-start",),
                "verbose_name": "timesheet period",
                "verbose_name_plural": "timesheet periods",
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name="Course",
            fields=[
                (
                    "subject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="leprikon.Subject",
                    ),
                ),
                ("unit", models.CharField(max_length=150, verbose_name="unit")),
                (
                    "periods",
                    models.ManyToManyField(
                        related_name="courses", to="leprikon.SchoolYearPeriod", verbose_name="periods"
                    ),
                ),
            ],
            options={
                "ordering": ("name",),
                "verbose_name": "course",
                "verbose_name_plural": "courses",
            },
            bases=("leprikon.subject",),
        ),
        migrations.CreateModel(
            name="CourseRegistration",
            fields=[
                (
                    "subjectregistration_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="leprikon.SubjectRegistration",
                    ),
                ),
            ],
            options={
                "verbose_name": "course registration",
                "verbose_name_plural": "course registrations",
            },
            bases=("leprikon.subjectregistration",),
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "subject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="leprikon.Subject",
                    ),
                ),
                ("start_date", models.DateField(verbose_name="start date")),
                ("end_date", models.DateField(verbose_name="end date")),
                ("start_time", models.TimeField(blank=True, null=True, verbose_name="start time")),
                ("end_time", models.TimeField(blank=True, null=True, verbose_name="end time")),
            ],
            options={
                "ordering": ("start_date", "start_time"),
                "verbose_name": "event",
                "verbose_name_plural": "events",
            },
            bases=("leprikon.subject",),
        ),
        migrations.CreateModel(
            name="EventRegistration",
            fields=[
                (
                    "subjectregistration_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="leprikon.SubjectRegistration",
                    ),
                ),
            ],
            options={
                "verbose_name": "event registration",
                "verbose_name_plural": "event registrations",
            },
            bases=("leprikon.subjectregistration",),
        ),
        migrations.AddField(
            model_name="timesheetentry",
            name="entry_type",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="entries",
                to="leprikon.TimesheetEntryType",
                verbose_name="entry type",
            ),
        ),
        migrations.AddField(
            model_name="timesheetentry",
            name="timesheet",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="timesheet_entries",
                to="leprikon.Timesheet",
                verbose_name="timesheet",
            ),
        ),
        migrations.AddField(
            model_name="timesheet",
            name="period",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="timesheets",
                to="leprikon.TimesheetPeriod",
                verbose_name="period",
            ),
        ),
        migrations.AddField(
            model_name="subjectregistration",
            name="participant_age_group",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="leprikon.AgeGroup",
                verbose_name="age group",
            ),
        ),
        migrations.AddField(
            model_name="subjectregistration",
            name="participant_insurance",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="leprikon.Insurance",
                verbose_name="insurance",
            ),
        ),
        migrations.AddField(
            model_name="subjectregistration",
            name="participant_school",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="leprikon.School",
                verbose_name="school",
            ),
        ),
        migrations.AddField(
            model_name="subjectregistration",
            name="subject",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="registrations",
                to="leprikon.Subject",
                verbose_name="subject",
            ),
        ),
        migrations.AddField(
            model_name="subjectregistration",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="leprikon_registrations",
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
        migrations.AddField(
            model_name="subjectpayment",
            name="registration",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="payments",
                to="leprikon.SubjectRegistration",
                verbose_name="registration",
            ),
        ),
        migrations.AddField(
            model_name="subjectpayment",
            name="related_payment",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="related_payments",
                to="leprikon.SubjectPayment",
                verbose_name="related payment",
            ),
        ),
        migrations.AddField(
            model_name="subjectgroup",
            name="subject_types",
            field=models.ManyToManyField(related_name="groups", to="leprikon.SubjectType", verbose_name="subject type"),
        ),
        migrations.AddField(
            model_name="subjectattachment",
            name="subject",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="attachments",
                to="leprikon.Subject",
                verbose_name="subject",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="age_groups",
            field=models.ManyToManyField(
                blank=True, related_name="subjects", to="leprikon.AgeGroup", verbose_name="age groups"
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="groups",
            field=models.ManyToManyField(
                blank=True, related_name="subjects", to="leprikon.SubjectGroup", verbose_name="groups"
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="leaders",
            field=models.ManyToManyField(
                blank=True, related_name="subjects", to="leprikon.Leader", verbose_name="leaders"
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="page",
            field=cms.models.fields.PageField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="cms.Page",
                verbose_name="page",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="photo",
            field=filer.fields.image.FilerImageField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="filer.Image",
                verbose_name="photo",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="place",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="subjects",
                to="leprikon.Place",
                verbose_name="place",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="questions",
            field=models.ManyToManyField(
                blank=True,
                help_text="Add additional questions to be asked in the registration form.",
                related_name="+",
                to="leprikon.Question",
                verbose_name="additional questions",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="reg_print_setup",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="leprikon.PrintSetup",
                verbose_name="registration print setup",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="school_year",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="subjects",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="subject",
            name="subject_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="subjects",
                to="leprikon.SubjectType",
                verbose_name="subject type",
            ),
        ),
        migrations.AddField(
            model_name="participant",
            name="school",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="participants",
                to="leprikon.School",
                verbose_name="school",
            ),
        ),
        migrations.AddField(
            model_name="participant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="leprikon_participants",
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
        migrations.AddField(
            model_name="leprikonsite",
            name="bill_print_setup",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="leprikon.PrintSetup",
                verbose_name="bill print setup",
            ),
        ),
        migrations.AddField(
            model_name="leprikonsite",
            name="reg_print_setup",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="leprikon.PrintSetup",
                verbose_name="registration print setup",
            ),
        ),
        migrations.AddField(
            model_name="leaderlistplugin",
            name="school_year",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="leaderlistplugin",
            name="subject",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.Subject",
                verbose_name="subject",
            ),
        ),
        migrations.AddField(
            model_name="leader",
            name="school_years",
            field=models.ManyToManyField(related_name="leaders", to="leprikon.SchoolYear", verbose_name="school years"),
        ),
        migrations.AddField(
            model_name="leader",
            name="user",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="leprikon_leader",
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
        migrations.AddField(
            model_name="filteredleaderlistplugin",
            name="school_year",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="filteredeventlistplugin",
            name="event_types",
            field=models.ManyToManyField(
                related_name="+",
                to="leprikon.SubjectType",
                verbose_name="event types",
            ),
        ),
        migrations.AddField(
            model_name="filteredeventlistplugin",
            name="school_year",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="filteredcourselistplugin",
            name="course_types",
            field=models.ManyToManyField(
                related_name="+",
                to="leprikon.SubjectType",
                verbose_name="course types",
            ),
        ),
        migrations.AddField(
            model_name="filteredcourselistplugin",
            name="school_year",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="eventlistplugin",
            name="event_types",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by event types.",
                related_name="+",
                to="leprikon.SubjectType",
                verbose_name="event types",
            ),
        ),
        migrations.AddField(
            model_name="eventlistplugin",
            name="groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by groups.",
                related_name="+",
                to="leprikon.SubjectGroup",
                verbose_name="event groups",
            ),
        ),
        migrations.AddField(
            model_name="eventlistplugin",
            name="leaders",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by leaders.",
                related_name="+",
                to="leprikon.Leader",
                verbose_name="leaders",
            ),
        ),
        migrations.AddField(
            model_name="eventlistplugin",
            name="school_year",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="courselistplugin",
            name="course_types",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by course types.",
                related_name="+",
                to="leprikon.SubjectType",
                verbose_name="course types",
            ),
        ),
        migrations.AddField(
            model_name="courselistplugin",
            name="groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by groups.",
                related_name="+",
                to="leprikon.SubjectGroup",
                verbose_name="course groups",
            ),
        ),
        migrations.AddField(
            model_name="courselistplugin",
            name="leaders",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by leaders.",
                related_name="+",
                to="leprikon.Leader",
                verbose_name="leaders",
            ),
        ),
        migrations.AddField(
            model_name="courselistplugin",
            name="school_year",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.SchoolYear",
                verbose_name="school year",
            ),
        ),
        migrations.AddField(
            model_name="coursejournalleaderentry",
            name="timesheet",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="course_entries",
                to="leprikon.Timesheet",
                verbose_name="timesheet",
            ),
        ),
        migrations.AddField(
            model_name="coursediscount",
            name="period",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="discounts",
                to="leprikon.SchoolYearPeriod",
                verbose_name="period",
            ),
        ),
        migrations.AddField(
            model_name="contact",
            name="leader",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="contacts",
                to="leprikon.Leader",
                verbose_name="leader",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="timesheet",
            unique_together=set([("period", "leader")]),
        ),
        migrations.AlterUniqueTogether(
            name="subjectregistration",
            unique_together=set([("subject", "participant_birth_num")]),
        ),
        migrations.AlterUniqueTogether(
            name="participant",
            unique_together=set([("user", "birth_num")]),
        ),
        migrations.AlterUniqueTogether(
            name="messagerecipient",
            unique_together=set([("message", "recipient")]),
        ),
        migrations.AddField(
            model_name="eventplugin",
            name="event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, related_name="+", to="leprikon.Event", verbose_name="event"
            ),
        ),
        migrations.AddField(
            model_name="eventdiscount",
            name="registration",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="discounts",
                to="leprikon.EventRegistration",
                verbose_name="registration",
            ),
        ),
        migrations.AddField(
            model_name="coursetime",
            name="course",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="times",
                to="leprikon.Course",
                verbose_name="course",
            ),
        ),
        migrations.AddField(
            model_name="courseregistrationhistory",
            name="course",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="registrations_history",
                to="leprikon.Course",
                verbose_name="course",
            ),
        ),
        migrations.AddField(
            model_name="courseregistrationhistory",
            name="registration",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="course_history",
                to="leprikon.CourseRegistration",
                verbose_name="course",
            ),
        ),
        migrations.AddField(
            model_name="courseplugin",
            name="course",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="leprikon.Course",
                verbose_name="course",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="coursejournalleaderentry",
            unique_together=set([("course_entry", "timesheet")]),
        ),
        migrations.AddField(
            model_name="coursejournalentry",
            name="course",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="journal_entries",
                to="leprikon.Course",
                verbose_name="course",
            ),
        ),
        migrations.AddField(
            model_name="coursejournalentry",
            name="registrations",
            field=models.ManyToManyField(
                blank=True,
                related_name="journal_entries",
                to="leprikon.CourseRegistration",
                verbose_name="participants",
            ),
        ),
        migrations.AddField(
            model_name="coursediscount",
            name="registration",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="discounts",
                to="leprikon.CourseRegistration",
                verbose_name="registration",
            ),
        ),
    ]



from django.contrib.auth import get_user_model
from rest_framework import serializers

from leprikon.models.activities import Activity, CalendarEvent, RegistrationParticipant
from leprikon.models.calendar import CalendarExport
from leprikon.models.schoolyear import SchoolYear


class RegistrationParticipantSerializer(serializers.ModelSerializer):
    class Meta:
        model = RegistrationParticipant
        fields = ["id", "label"]


class SchoolYearSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = SchoolYear
        fields = ["id", "name", "year", "active"]


class SetSchoolYearSerializer(serializers.Serializer):
    id = serializers.IntegerField()


class CredentialsSerializer(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField()


class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = get_user_model()
        fields = ["username", "first_name", "last_name", "email"]


class ActivitySerializer(serializers.ModelSerializer):
    class Meta:
        model = Activity
        fields = ["id", "name"]


class GetResourceConflictSerializer(serializers.Serializer):
    start = serializers.DateTimeField()
    end = serializers.DateTimeField()


class ResourceConflictSerializer(serializers.Serializer):
    id = serializers.CharField()
    start = serializers.DateTimeField()
    end = serializers.DateTimeField()
    title = serializers.CharField()
    allDay = serializers.BooleanField()
    color = serializers.CharField()


class CalendarEventSerializer(serializers.ModelSerializer):

    class Meta:
        model = CalendarEvent
        fields = [
            "id",
            "name",
            "start",
            "end",
        ]


class CalendarExportSerializer(serializers.ModelSerializer):
    class Meta:
        model = CalendarExport
        fields = "__all__"



from cms.plugin_pool import plugin_pool
from django.utils.translation import gettext as _

from ..models.events import EventListPlugin, EventPlugin, FilteredEventListPlugin
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonEventPlugin(LeprikonPluginBase):
    name = _("Event")
    model = EventPlugin
    raw_id_fields = ("event",)

    def get_render_template(self, context, instance, placeholder):
        return "leprikon/cms/event/%s.html" % instance.template


@plugin_pool.register_plugin
class LeprikonEventListPlugin(LeprikonPluginBase):
    name = _("Event list")
    model = EventListPlugin
    filter_horizontal = ("age_groups", "target_groups", "groups", "leaders")

    def get_render_template(self, context, instance, placeholder):
        return "leprikon/cms/event_list/%s.html" % instance.template


@plugin_pool.register_plugin
class LeprikonFilteredEventListPlugin(LeprikonPluginBase):
    name = _("Event list with search form")
    model = FilteredEventListPlugin
    render_template = "leprikon/cms/event_list_filtered.html"



from django.db import models
from django.utils.translation import gettext_lazy as _

from ..utils import first_upper
from .statgroup import StatGroup


class AgeGroup(models.Model):
    name = models.CharField(_("name"), max_length=50)
    min_age = models.PositiveSmallIntegerField(_("minimal age"), blank=True, null=True)
    max_age = models.PositiveSmallIntegerField(_("maximal age"), blank=True, null=True)
    order = models.IntegerField(_("order"), blank=True, default=0)
    require_school = models.BooleanField(_("require school"), default=True)
    stat_group = models.ForeignKey(
        StatGroup, on_delete=models.PROTECT, related_name="+", verbose_name=_("group for statistics")
    )

    class Meta:
        app_label = "leprikon"
        ordering = ("order",)
        verbose_name = _("age group")
        verbose_name_plural = _("age groups")

    def __str__(self):
        return self.name

    def title(self):
        return first_upper(self.name)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-08-28 09:23
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0027_journals'),
    ]

    operations = [
        migrations.AddField(
            model_name='agegroup',
            name='require_school',
            field=models.BooleanField(default=True, verbose_name='require school'),
        ),
    ]



from cms.plugin_pool import plugin_pool
from django.utils.translation import gettext as _

from ..models.courses import CourseListPlugin, CoursePlugin, FilteredCourseListPlugin
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonCoursePlugin(LeprikonPluginBase):
    name = _("Course")
    model = CoursePlugin
    raw_id_fields = ("course",)

    def get_render_template(self, context, instance, placeholder):
        return "leprikon/cms/course/%s.html" % instance.template


@plugin_pool.register_plugin
class LeprikonCourseListPlugin(LeprikonPluginBase):
    name = _("Course list")
    model = CourseListPlugin
    filter_horizontal = ("age_groups", "target_groups", "groups", "leaders")

    def get_render_template(self, context, instance, placeholder):
        return "leprikon/cms/course_list/%s.html" % instance.template


@plugin_pool.register_plugin
class LeprikonFilteredCourseListPlugin(LeprikonPluginBase):
    name = _("Course list with search form")
    model = FilteredCourseListPlugin
    render_template = "leprikon/cms/course_list_filtered.html"



import uuid
from datetime import date, datetime, time, timedelta
from functools import cached_property
from typing import TYPE_CHECKING, Optional

from django.db import models
from django.urls import reverse
from django.utils.formats import date_format, time_format
from django.utils.translation import gettext_lazy as _
from icalendar import Calendar, Event

from leprikon.models.leprikonsite import LeprikonSite
from leprikon.utils.calendar import SimpleEvent, TimeSlot, WeeklyTimes

from .fields import DaysOfWeek, DaysOfWeekField
from .roles import Leader
from .startend import StartEndMixin
from .times import WeeklyTime

if TYPE_CHECKING:
    from .activities import Activity  # Avoid circular import


class Resource(models.Model):
    name = models.CharField(_("name"), max_length=255)
    leader = models.OneToOneField(
        Leader,
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name="resource",
        verbose_name=_("leader"),
    )

    class Meta:
        app_label = "leprikon"
        verbose_name = _("resource")
        verbose_name_plural = _("resources")

    def __str__(self):
        return self.name

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        if self.leader:
            self.name = str(self.leader)
        return super().save(force_insert, force_update, using, update_fields)

    @property
    def all_availabilities(self) -> list["ResourceAvailability"]:
        return list(self.availabilities.all())

    @property
    def weekly_times(self) -> WeeklyTimes:
        return WeeklyTimes(availability.weekly_time for availability in self.all_availabilities)


class ResourceAvailability(StartEndMixin, models.Model):
    resource = models.ForeignKey(
        Resource, on_delete=models.CASCADE, related_name="availabilities", verbose_name=_("resource")
    )
    start_date = models.DateField(_("start date"), blank=True, null=True)
    end_date = models.DateField(_("end date"), blank=True, null=True)
    days_of_week: DaysOfWeek = DaysOfWeekField(_("days of week"))
    start_time: time = models.TimeField(_("start time"), default=time(8, 0))
    end_time: time = models.TimeField(_("end time"), default=time(16, 0))

    class Meta:
        app_label = "leprikon"
        ordering = ("start_time",)
        verbose_name = _("availability")
        verbose_name_plural = _("availabilities")

    @property
    def weekly_time(self) -> WeeklyTime:
        return WeeklyTime(
            start_date=self.start_date,
            end_date=self.end_date,
            days_of_week=self.days_of_week,
            start_time=self.start_time,
            end_time=self.end_time,
        )

    def __str__(self):
        return str(self.weekly_time)

    def __and__(self, other: "ResourceAvailability") -> Optional[WeeklyTime]:
        return self.weekly_time & other.weekly_time


class ResourceGroup(models.Model):
    name = models.CharField(_("name"), max_length=255)
    resources = models.ManyToManyField(Resource, related_name="groups", verbose_name=_("resources"))

    class Meta:
        app_label = "leprikon"
        verbose_name = _("resource group")
        verbose_name_plural = _("resource groups")

    def __str__(self):
        return self.name


class CalendarEvent(models.Model):
    name = models.CharField(_("name"), max_length=255)
    activity: Optional["Activity"] = models.ForeignKey(
        "leprikon.Activity",
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name="calendar_events",
        verbose_name=_("activity"),
    )
    start_date = models.DateField(_("start date"))
    start_time = models.TimeField(_("start time"), blank=True, null=True)
    end_date = models.DateField(_("end date"))
    end_time = models.TimeField(_("end time"), blank=True, null=True)
    preparation_time = models.DurationField(
        _("preparation time"),
        default=timedelta(0),
        help_text=_("Time to prepare before the event. (HH:MM:SS)"),
    )
    recovery_time = models.DurationField(
        _("recovery time"),
        default=timedelta(0),
        help_text=_("Time to recover after the event. (HH:MM:SS)"),
    )
    blocks_all_resources = models.BooleanField(
        _("blocks all resources"),
        default=False,
        help_text=_("If checked, the event will block all resources, not just the selected ones."),
    )
    resources = models.ManyToManyField(
        Resource, blank=True, related_name="calendar_events", verbose_name=_("resources")
    )
    resource_groups = models.ManyToManyField(
        ResourceGroup, blank=True, related_name="calendar_events", verbose_name=_("resource groups")
    )
    is_canceled = models.BooleanField(_("is canceled"), default=False)

    @property
    def start(self) -> datetime:
        return datetime.combine(self.start_date, time(0) if self.start_time is None else self.start_time)

    @property
    def end(self) -> datetime:
        return datetime.combine(self.end_date, time(0) if self.end_time is None else self.end_time)

    @cached_property
    def effective_start(self) -> datetime:
        if self.start_time:
            return datetime.combine(self.start_date, self.start_time) - self.preparation_time
        return datetime.combine(self.start_date, time(0))

    @cached_property
    def effective_end(self) -> datetime:
        if self.end_time:
            return datetime.combine(self.end_date, self.end_time) + self.recovery_time
        return datetime.combine(self.end_date, time(0))

    @property
    def timeslot(self) -> TimeSlot:
        return TimeSlot(self.effective_start, self.effective_end)

    @property
    def simple_event(self) -> SimpleEvent:
        return SimpleEvent(
            timeslot=self.timeslot,
            resource_groups=[{resource.id} for resource in self.resources.all()]
            + [
                {resource.id for resource in resource_group.resources.all()}
                for resource_group in self.resource_groups.all()
            ],
        )

    class Meta:
        app_label = "leprikon"
        verbose_name = _("calendar event")
        verbose_name_plural = _("calendar events")
        ordering = ("start_date", "start_time")

    def __str__(self):
        return "{start} - {end}".format(
            start=(
                date_format(self.start, "SHORT_DATETIME_FORMAT")
                if self.start_time
                else date_format(self.start_date, "SHORT_DATE_FORMAT")
            ),
            end=(
                (
                    date_format(self.end, "SHORT_DATETIME_FORMAT")
                    if self.end_date != self.start_date
                    else time_format(self.end_time)
                )
                if self.end_time
                else date_format(self.end_date, "SHORT_DATE_FORMAT")
            ),
        )

    @property
    def url(self) -> str:
        leprikon_site = LeprikonSite.objects.get_current()
        uri = reverse("admin:leprikon_calendarevent_change", args=(self.pk,))
        return leprikon_site.url + uri


class CalendarExport(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(_("name"), max_length=255)
    resources = models.ManyToManyField(
        Resource,
        blank=True,
        related_name="calendar_exports",
        verbose_name=_("resources"),
        help_text=_("Select resources to export calendar events for. Leave empty to export events for all resources."),
    )
    export_past_days = models.PositiveSmallIntegerField(_("export past days"), default=30)
    limit_events_count = models.PositiveSmallIntegerField(
        _("limit events count"),
        default=1000,
        help_text=_("Limit the number of events to export. Huge exports may fail to import to Your calendar."),
    )

    class Meta:
        app_label = "leprikon"
        verbose_name = _("calendar export")
        verbose_name_plural = _("calendar exports")

    def __str__(self):
        return self.name

    @cached_property
    def resource_ids(self) -> set[int]:
        return set(self.resources.values_list("id", flat=True))

    @property
    def relevant_events(self) -> models.QuerySet[CalendarEvent]:
        qs = CalendarEvent.objects.filter(end_date__gte=date.today() - timedelta(days=self.export_past_days))
        if self.resource_ids:
            qs = qs.filter(
                models.Q(blocks_all_resources=True)
                | models.Q(resources__in=self.resource_ids)
                | models.Q(resource_groups__resources__in=self.resource_ids)
            )
        return qs

    def get_ical(self) -> str:
        calendar = Calendar()
        calendar.add("prodid", "-//Leprikon//Calendar Export//EN")
        calendar.add("version", "2.0")
        for event in self.relevant_events[: self.limit_events_count]:
            ical_event = Event()
            ical_event.add("summary", event.name)
            ical_event.add("dtstart", event.start if event.start_time else event.start_date)
            # end date is exclusive if end time is not set
            ical_event.add("dtend", event.end if event.end_time else event.end_date + timedelta(days=1))
            ical_event.add("url", event.url)
            calendar.add_component(ical_event)
        return calendar.to_ical().decode("utf-8")



from django.db import models
from django.utils.translation import gettext_lazy as _

from ..utils import first_upper


class StatGroup(models.Model):
    name = models.CharField(_("name"), max_length=50)
    order = models.IntegerField(_("order"), blank=True, default=0)

    class Meta:
        app_label = "leprikon"
        ordering = ("order",)
        verbose_name = _("group for statistics")
        verbose_name_plural = _("groups for statistics")

    def __str__(self):
        return self.name

    def title(self):
        return first_upper(self.name)



from django.utils.translation import gettext_lazy as _

from ..utils import attributes


class SendMailAdminMixin:
    actions = ("send_mail",)

    @attributes(short_description=_("Send selected items by e-mail"))
    def send_mail(self, request, queryset):
        for obj in queryset.all():
            obj.send_mail()
        self.message_user(request, _("Selected items were sent by e-mail."))



from django.urls import reverse_lazy as reverse
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from ..forms.messages import MessageFilterForm
from ..models.messages import MessageRecipient
from .generic import BackViewMixin, DetailView, FilteredListView


class MessageListView(FilteredListView):
    model = MessageRecipient
    form_class = MessageFilterForm
    preview_template = "leprikon/message_preview.html"
    template_name = "leprikon/message_list.html"
    message_empty = _("No messages matching given query.")
    paginate_by = 10

    def get_title(self):
        return _("Messages")

    def get_form(self):
        return self.form_class(
            user=self.request.user,
            data=self.request.GET,
        )


class MessageDetailView(BackViewMixin, DetailView):
    back_url = reverse("leprikon:message_list")
    model = MessageRecipient

    def get_object(self):
        obj = super().get_object()
        if obj.viewed is None:
            obj.viewed = now()
            obj.save()
        return obj



import locale

import icu
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.http import Http404
from django.template.response import TemplateResponse
from django.utils.translation import gettext_lazy as _

from . import (  # NOQA
    account,
    activities,
    agegroup,
    agreements,
    calendar,
    citizenship,
    courses,
    department,
    donation,
    events,
    journals,
    leprikonsite,
    messages,
    orderables,
    organizations,
    place,
    printsetup,
    question,
    refundrequest,
    registrationlink,
    roles,
    school,
    schoolyear,
    statgroup,
    targetgroup,
    timesheets,
    transaction,
    user,
)

User = get_user_model()

admin.site.site_header = _("Leprikon administration")
admin.site.site_title = _("Leprikon administration")


def app_index(self, request, app_label, extra_context=None):
    app_dict = self._build_app_dict(request, app_label)
    if not app_dict:
        raise Http404("The requested admin page does not exist.")
    # Sort the models alphabetically within each app.
    collator = icu.Collator.createInstance(icu.Locale(".".join(locale.getlocale())))
    app_dict["models"].sort(key=lambda x: collator.getSortKey(x["name"].lower()))
    context = dict(
        self.each_context(request),
        title=_("Leprikon administration"),
        app_list=[app_dict],
        app_label=app_label,
    )
    context.update(extra_context or {})

    request.current_app = self.name

    return TemplateResponse(
        request, self.app_index_template or ["admin/%s/app_index.html" % app_label, "admin/app_index.html"], context
    )


def get_app_list(self, request):
    """
    Returns a sorted list of all the installed apps that have been
    registered in this site.
    """
    collator = icu.Collator.createInstance(icu.Locale(".".join(locale.getlocale())))

    app_dict = self._build_app_dict(request)

    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: collator.getSortKey(x["name"].lower()))

    # Sort the models alphabetically within each app.
    for app in app_list:
        app["models"].sort(key=lambda x: collator.getSortKey(x["name"].lower()))

    return app_list


# override default Admin site's app_index and get_app_list
admin.sites.AdminSite.app_index = app_index
admin.sites.AdminSite.get_app_list = get_app_list



from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from ..utils import currency
from .printsetup import PrintSetup
from .transaction import Transaction


class Donation(Transaction):
    object_name = "donation"
    transaction_types = Transaction.DONATIONS

    class Meta:
        proxy = True
        verbose_name = _("donation")
        verbose_name_plural = _("donations")

    def __str__(self):
        return f"{self.donor}, {currency(self.amount)}"

    @cached_property
    def all_recipients(self):
        return [self.donor]

    def get_print_setup(self, event):
        return self.organization.donation_print_setup or PrintSetup()



from bankreader.readers import register_reader
from bankreader.readers.best import BestReader
from bankreader.readers.csv import CsvReader
from bankreader.readers.gpc import GpcReader
from bankreader.readers.mt940 import MT940Reader


@register_reader
class CsobGpcReader(GpcReader):
    label = "ČSOB, GPC"
    encoding = "cp1250"


@register_reader
class KbBestReader(BestReader):
    label = "Komerční banka, Best"


@register_reader
class KbCsvReader(CsvReader):
    label = "Komerční banka, CSV"
    encoding = "cp1250"
    delimiter = ";"
    column_mapping = {
        "accounted_date": "Datum splatnosti",
        "entry_date": "Datum odepsání z jiné banky",
        "remote_account_number": "Protiúčet a kód banky",
        "remote_account_name": "Název protiúčtu",
        "amount": "Částka",
        "variable_symbol": "VS",
        "constant_symbol": "KS",
        "specific_symbol": "SS",
        "transaction_id": "Identifikace transakce",
        "sender_description": "Popis příkazce",
        "recipient_description": "Popis pro příjemce",
    }
    date_format = "%d.%m.%Y"
    decimal_separator = ","


@register_reader
class CsCsvReader(CsvReader):
    label = "Česká spořitelna, CSV"
    encoding = "utf16"
    column_mapping = {
        "accounted_date": "Datum zaúčtování",
        "entry_date": "Datum zaúčtování",
        "remote_account_number": "Protiúčet",
        "remote_account_name": "Název protiúčtu",
        "amount": "Částka",
        "variable_symbol": "Variabilní symbol",
        # "constant_symbol": "",
        # "specific_symbol": "",
        "transaction_id": "ID transakce",
        "sender_description": "Zpráva pro mě",
        "recipient_description": "Zpráva pro příjemce",
    }
    date_format = "%d.%m.%Y"
    decimal_separator = ","


@register_reader
class UniCreditReader(MT940Reader):
    label = "UniCredit Bank, MT940 (.sta)"



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-13 07:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_pays', '0001_initial'),
        ('leprikon', '0025_agreements'),
    ]

    operations = [
        migrations.AddField(
            model_name='leprikonsite',
            name='payment_gateway',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='django_pays.Gateway', verbose_name='Payment Gateway'),
        ),
        migrations.AddField(
            model_name='subjectpayment',
            name='pays_payment',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='django_pays.Payment', verbose_name='online payment'),
        ),
    ]



from django.db.models import F
from django.utils.translation import gettext_lazy as _

from ..forms.leaders import LeaderFilterForm
from ..models.roles import Leader
from .generic import DetailView, FilteredListView


class LeaderListView(FilteredListView):
    model = Leader
    form_class = LeaderFilterForm
    preview_template = "leprikon/leader_preview.html"
    template_name = "leprikon/leader_list.html"
    message_empty = _("No leaders matching given query.")
    paginate_by = 10

    def get_title(self):
        return _("Leaders in school year {}").format(self.request.school_year)

    def get_form(self):
        return self.form_class(
            school_year=self.request.school_year,
            data=self.request.GET,
        )


class LeaderDetailView(DetailView):
    model = Leader

    def get_queryset(self):
        return super().get_queryset().annotate(slug=F("user__username"))



from django.utils.translation import gettext_lazy as _

from ..forms.schoolyear import SchoolYearForm
from .generic import FormView


class SchoolYearView(FormView):
    form_class = SchoolYearForm
    title = _("Switch school year")

    def get_message(self):
        return _("You now work with school year {}.").format(self.request.school_year)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["request"] = self.request
        return kwargs

    def form_valid(self, form):
        self.request.school_year = form.cleaned_data["school_year"]
        return super().form_valid(form)



from datetime import date

from django.contrib.auth import authenticate, login, logout
from django.http import HttpResponse
from django.utils.translation import gettext_lazy as _
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import OpenApiParameter, extend_schema
from rest_framework import status, viewsets
from rest_framework.decorators import action
from rest_framework.exceptions import AuthenticationFailed, NotFound
from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated
from rest_framework.request import Request
from rest_framework.response import Response

from leprikon.models.calendar import CalendarExport

from ..models.activities import ActivityVariant, CalendarEvent
from ..models.journals import Journal
from ..models.schoolyear import SchoolYear
from .serializers import (
    ActivitySerializer,
    CalendarEventSerializer,
    CalendarExportSerializer,
    CredentialsSerializer,
    GetResourceConflictSerializer,
    RegistrationParticipantSerializer,
    ResourceConflictSerializer,
    SchoolYearSerializer,
    SetSchoolYearSerializer,
    UserSerializer,
)


class JournalViewSet(viewsets.GenericViewSet):
    def get_queryset(self):
        if self.request.user.is_staff:
            return Journal.objects.all()
        return Journal.objects.filter(leaders=self.request.leader)

    @extend_schema(
        responses={200: RegistrationParticipantSerializer(many=True)},
        methods=["get"],
        parameters=[OpenApiParameter(name="date", type=OpenApiTypes.DATE)],
    )
    @action(detail=True, methods=["get"], permission_classes=[IsAuthenticated])
    def participants(self, request, pk: str):
        journal: Journal = self.get_object()
        try:
            d = date.fromisoformat(request.GET["date"])
        except (KeyError, ValueError):
            d = date.today()

        participants = journal.get_valid_participants(d)

        return Response(RegistrationParticipantSerializer(participants, many=True).data)


class SchoolYearViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = SchoolYearSerializer

    def get_queryset(self):
        if self.request.user and self.request.user.is_staff:
            return SchoolYear.objects.all()
        return SchoolYear.objects.filter(active=True)

    @extend_schema(request=None, responses={200: SchoolYearSerializer}, methods=["get"])
    @extend_schema(request=SetSchoolYearSerializer, responses={204: None}, methods=["post"])
    @action(detail=False, methods=["get", "post"])
    def current(self, request):
        if request.method == "GET":
            return Response(SchoolYearSerializer(request.school_year).data)

        if request.method == "POST":
            serializer = SetSchoolYearSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)
            try:
                request._request.school_year = self.get_queryset().get(pk=serializer.validated_data["id"])
                return Response(status=status.HTTP_204_NO_CONTENT)
            except SchoolYear.DoesNotExist:
                raise NotFound


class UserViewSet(viewsets.ViewSet):
    @extend_schema(operation_id="user_login", request=CredentialsSerializer, responses={200: UserSerializer})
    @action(detail=False, methods=["post"])
    def login(self, request):
        serializer = CredentialsSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        if user := authenticate(request, **serializer.validated_data):
            login(request, user)
            return Response(UserSerializer(user).data)
        else:
            raise AuthenticationFailed()

    @extend_schema(responses={200: UserSerializer})
    @action(detail=False, methods=["get"], permission_classes=[IsAuthenticated])
    def me(self, request: Request) -> Response:
        return Response(UserSerializer(request.user).data)

    @extend_schema(operation_id="user_logout", responses={204: None})
    @action(
        detail=False,
        methods=["post"],
        permission_classes=[IsAuthenticated],
    )
    def logout(self, request: Request) -> Response:
        logout(request)
        return Response(status=status.HTTP_204_NO_CONTENT)


class ActivityViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = ActivitySerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        queryset = ActivityVariant.objects.filter(activity__school_year=self.request.school_year)
        if not self.request.user.is_staff:
            queryset = queryset.filter(activity__is_visible=True)
        return queryset

    @extend_schema(
        operation_id="resource_conflicts",
        request=None,
        responses={200: ResourceConflictSerializer(many=True)},
        methods=["get"],
        parameters=[
            OpenApiParameter(name="start", type=OpenApiTypes.DATETIME),
            OpenApiParameter(name="end", type=OpenApiTypes.DATETIME),
        ],
    )
    @action(
        detail=True,
        permission_classes=[IsAuthenticated],
    )
    def resource_conflicts(self, request: Request, pk: str):
        """
        Returns a list of calendar events that use the same resources as the activity variant.
        """
        activity_variant: ActivityVariant = self.get_object()
        input_serializer = GetResourceConflictSerializer(data=request.query_params)
        input_serializer.is_valid(raise_exception=True)

        conflicts = [
            dict(
                id=f"{conflicting_timeslot.start}-{conflicting_timeslot.end}",
                start=conflicting_timeslot.start,
                end=conflicting_timeslot.end,
                title=_("unavailable time"),
                allDay=False,
                color="#99e",  # TODO: Use configurable color
            )
            for conflicting_timeslot in activity_variant.get_conflicting_timeslots(
                start_date=input_serializer.validated_data["start"].date(),
                end_date=input_serializer.validated_data["end"].date(),
            )
        ]

        return Response(
            ResourceConflictSerializer(
                conflicts,
                many=True,
            ).data
        )


class CalendarEventViewSet(viewsets.ModelViewSet):
    serializer_class = CalendarEventSerializer
    permission_classes = [DjangoModelPermissions]

    def get_queryset(self):
        return CalendarEvent.objects.all()


class CalendarExportViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = CalendarExportSerializer
    permission_classes = [DjangoModelPermissions]

    def get_queryset(self):
        return CalendarExport.objects.all()

    @extend_schema(operation_id="calendar_export_ical", responses={200: str})
    @action(detail=True, methods=["get"], permission_classes=[])
    def ical(self, request: Request, pk: str):
        calendar_export: CalendarExport = self.get_object()
        return HttpResponse(
            calendar_export.get_ical(),
            content_type="text/calendar",
            headers={"Content-Disposition": f'inline; filename="calendar-{calendar_export.id}.ics"'},
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2022-01-04 16:35
from __future__ import unicode_literals

from django.db import migrations
import django.db.models.deletion
import leprikon.models.fields


def add_subject_type_page(apps, schema_editor):
    SubjectType = apps.get_model("leprikon", "SubjectType")
    Page = apps.get_model("cms", "Page")

    for subject_type in SubjectType.objects.all():
        subject_type.page = Page.objects.filter(
            application_urls="LeprikonSubjectTypeApp",
            application_namespace=subject_type.slug,
            publisher_is_draft=True,
        ).first()
        if subject_type.page:
            subject_type.save()


class Migration(migrations.Migration):

    dependencies = [
        ("cms", "0022_auto_20180620_1551"),
        ("leprikon", "0064_journals_cleanup"),
    ]

    operations = [
        migrations.AddField(
            model_name="subjecttype",
            name="page",
            field=leprikon.models.fields.UniquePageField(
                blank=True,
                help_text="Select page, where the activities will be listed. New activity type requires a new page. If You assign an existing page, the current content of the page will be replaced with the new list of activities.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="leprikon_activity_type",
                to="cms.Page",
                verbose_name="page",
            ),
            preserve_default=False,
        ),
        migrations.RunPython(add_subject_type_page),
    ]



from dataclasses import dataclass
from datetime import date, datetime, time, timedelta
from typing import Optional

from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from ..utils import attributes, comma_separated
from ..utils.calendar import WeeklyTime, time_slot_format
from .fields import DaysOfWeek, DaysOfWeekField
from .startend import StartEndMixin


@dataclass
class Time:
    date: date
    start: Optional[time]
    end: Optional[time]

    def _comparable(self) -> "tuple[date, time, time]":
        return (self.date, self.start or time(0), self.end or time(0))

    def __lt__(self, other: "Time") -> bool:
        return self._comparable() < other._comparable()


class AbstractTime(StartEndMixin, models.Model):
    start_date = models.DateField(
        _("start date"),
        blank=True,
        null=True,
        help_text=_("If not set, the time is valid for all dates."),
    )
    end_date = models.DateField(
        _("end date"),
        blank=True,
        null=True,
        help_text=_("If not set, the time is valid for all dates."),
    )
    days_of_week: DaysOfWeek = DaysOfWeekField(_("days of week"))
    start_time: time = models.TimeField(_("start time"), blank=True, null=True)
    end_time: time = models.TimeField(
        _("end time"),
        blank=True,
        null=True,
        help_text=_("If end time is 0:00:00, it means the end of the day (effectively 24:00:00)."),
    )

    class Meta:
        abstract = True

    def __str__(self):
        if self.start_time is not None and self.end_time is not None:
            return _("{days}, {time}").format(
                days=self.days_of_week,
                time=time_slot_format(self.start_time, self.end_time),
            )
        elif self.start_time is not None:
            return _("{days}, {time}").format(
                days=self.days_of_week,
                time=self.start_time.strftime("%H:%M"),
            )
        else:
            return str(self.days_of_week)

    @property
    def weekly_time(self) -> WeeklyTime:
        return WeeklyTime(
            start_date=self.start_date,
            end_date=self.end_date,
            days_of_week=self.days_of_week,
            start_time=time(0) if self.start_time is None else self.start_time,
            end_time=time(0) if self.end_time is None else self.end_time,
        )

    @property
    def delta(self):
        if self.start_time is not None and self.end_time is not None:
            d = date(1, 1, 1)
            return datetime.combine(d, self.end_time) - datetime.combine(d, self.start_time)
        else:
            return timedelta(0)

    def get_next_time(self, now: date | datetime = None):
        now = now or datetime.now()
        now_day = now.isoweekday()
        # If the start time is None or less than now, we want to move to the next day
        force_next_day = isinstance(now, date) or self.start_time is None or self.start_time <= now.time()
        if force_next_day:
            now_day = (now_day + 1) % 7
        daydelta = min((d.isoweekday() - now_day) % 7 for d in self.days_of_week)
        if daydelta == 0 and force_next_day:
            daydelta = 7
        if isinstance(now, datetime):
            next_date = now.date() + timedelta(daydelta)
        else:
            next_date = now + timedelta(daydelta)
        return Time(
            date=next_date,
            start=self.start_time,
            end=self.end_time,
        )


class TimesMixin:
    @cached_property
    def all_times(self):
        return list(self.times.all())

    @attributes(short_description=_("times"))
    def get_times_list(self):
        return comma_separated(self.all_times)

    def get_next_time(self, now=None):
        try:
            return min(t.get_next_time(now) for t in self.all_times)
        except ValueError:
            return None



from django.contrib import admin
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from leprikon.models.leprikonsite import LeprikonSite

from ..models.calendar import CalendarEvent, CalendarExport, Resource, ResourceAvailability, ResourceGroup
from .filters import IsNullFieldListFilter


class AvailabilityInlineAdmin(admin.TabularInline):
    model = ResourceAvailability
    extra = 0


@admin.register(Resource)
class ResourceAdmin(admin.ModelAdmin):
    inlines = (AvailabilityInlineAdmin,)
    search_fields = ("name",)
    list_display = ("name", "availabilities", "leader")
    list_filter = (("leader", IsNullFieldListFilter),)
    raw_id_fields = ("leader",)

    @admin.display(description=_("availabilities"))
    def availabilities(self, obj: Resource) -> str:
        return mark_safe("<br/>".join(str(availability) for availability in obj.availabilities.all()))


@admin.register(ResourceGroup)
class ResourceGroupAdmin(admin.ModelAdmin):
    search_fields = ("name",)
    list_display = ("name", "resources_list")
    filter_horizontal = ("resources",)

    @admin.display(description=_("resources"))
    def resources_list(self, obj: ResourceGroup) -> str:
        return mark_safe("<br/>".join(str(resource) for resource in obj.resources.all()))


@admin.register(CalendarEvent)
class CalendarEventAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "activity",
        "start_date",
        "start_time",
        "resources_list",
        "resource_groups_list",
    )
    list_filter = (
        "activity__activity_type",
        "resources",
        "resource_groups",
    )
    raw_id_fields = ("activity",)
    filter_horizontal = ("resources", "resource_groups")
    search_fields = ("activity_variant__name",)

    def get_queryset(self, request):
        return super().get_queryset(request).select_related("activity").prefetch_related("resources")

    @admin.display(description=_("resources"))
    def resources_list(self, obj):
        return ", ".join([str(resource) for resource in obj.resources.all()])

    @admin.display(description=_("resource groups"))
    def resource_groups_list(self, obj):
        return ", ".join([str(group) for group in obj.resource_groups.all()])


@admin.register(CalendarExport)
class CalendarExportAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "resources_list",
        "relevant_events_count",
        "ical_url",
    )
    list_filter = ("resources",)
    filter_horizontal = ("resources",)

    @admin.display(description=_("resources"))
    def resources_list(self, obj: CalendarExport) -> str:
        if not obj.resource_ids:
            return _("all resources")
        return mark_safe("<br>".join(str(resource) for resource in obj.resources.all()))

    @admin.display(description=_("relevant events count"))
    def relevant_events_count(self, obj: CalendarExport) -> str:
        return f"{obj.relevant_events.count()} / {obj.limit_events_count}"

    @admin.display(description=_("iCal URL"))
    def ical_url(self, obj: CalendarExport) -> str:
        leprikon_site = LeprikonSite.objects.get_current()
        uri = reverse("api:calendarexport-ical", args=(obj.id,))
        return leprikon_site.url + uri
